@@ -590,7 +590,7 @@ rmd160_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RMD160 ($Revision: 1.17.2.4 $)";
+const char * const gnupgext_version = "RMD160 ($Revision: 1.17.2.5 $)";
 
 static struct {
     int class;
@@ -625,7 +625,11 @@ gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
 	    ret = &func_table[i].value;
 	    break;
 	  default:
+#ifndef __riscos__
 	    ret = func_table[i].func;
+#else /* __riscos__ */
+	    ret = (void *) func_table[i].func;
+#endif /* __riscos__ */
 	    break;
 	}
 	i++;
