@@ -210,14 +210,14 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDEGD ($Revision: 1.7.2.4 $)";
+const char * const gnupgext_version = "RNDEGD ($Revision: 1.7.2.5 $)";
 
 static struct {
     int class;
     int version;
-    void *func;
+    int (*func)(void);
 } func_table[] = {
-    { 40, 1, gather_random },
+    { 40, 1, (int (*)(void))gather_random },
 };
 
 
