@@ -851,15 +851,15 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDUNIX ($Revision: 1.14.2.5 $)";
+const char * const gnupgext_version = "RNDUNIX ($Revision: 1.14.2.6 $)";
 
 
 static struct {
     int class;
     int version;
-    void *func;
+    int (*func)(void);
 } func_table[] = {
-    { 40, 1, gather_random },
+    { 40, 1, (int (*)(void))gather_random },
 };
 
 /****************
