@@ -67,6 +67,9 @@
 #include <string.h>
 
 #include <windows.h>
+#ifdef __CYGWIN32__
+# include <winioctl.h>
+#endif
 
 
 #include "types.h"
@@ -561,7 +564,8 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
 		log_debug ("rndw32#slow_gatherer_nt: check product options\n" );
 	    status = RegQueryValueEx (hKey, "ProductType", 0, NULL,
 				      szValue, &dwSize);
-	    if (status == ERROR_SUCCESS && stricmp (szValue, "WinNT")) {
+	    if (status == ERROR_SUCCESS
+                && ascii_strcasecmp (szValue, "WinNT")) {
 		/* Note: There are (at least) three cases for ProductType:
 		 * WinNT = NT Workstation, ServerNT = NT Server, LanmanNT =
 		 * NT Server acting as a Domain Controller */
@@ -729,7 +733,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 					  size_t length, int level )
 {
     static int is_initialized;
-    static int is_windows95;
+    static int is_windowsNT, has_toolhelp;
 
 
     if( !level )
@@ -747,7 +751,9 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 
 	GetVersionEx( &osvi );
 	platform = osvi.dwPlatformId;
-	is_windows95 = platform == VER_PLATFORM_WIN32_WINDOWS;
+        is_windowsNT = platform == VER_PLATFORM_WIN32_NT;
+        has_toolhelp = (platform == VER_PLATFORM_WIN32_WINDOWS
+                        || (is_windowsNT && osvi.dwMajorVersion >= 5));
 
 	if ( platform == VER_PLATFORM_WIN32s ) {
 	    g10_log_fatal("can't run on a W32s platform\n" );
@@ -762,11 +768,11 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	log_debug ("rndw32#gather_random: req=%d len=%u lvl=%d\n",
 			   requester, (unsigned int)length, level );
 
-    if (is_windows95 ) {
-	slow_gatherer_windows95( add, requester );
+    if ( has_toolhelp ) {
+        slow_gatherer_windows95 ( add, requester );
     }
-    else {
-	slow_gatherer_windowsNT( add, requester );
+    if ( is_windowsNT ) {
+        slow_gatherer_windowsNT ( add, requester );
     }
 
     return 0;
@@ -915,7 +921,7 @@ gather_random_fast( void (*add)(const void*, size_t, int), int requester )
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDW32 ($Revision: 1.1.4.8 $)";
+const char * const gnupgext_version = "RNDW32 ($Revision: 1.1.4.11 $)";
 
 static struct {
     int class;
