@@ -363,7 +363,7 @@ sha1_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "SHA1 ($Revision: 1.12.2.6 $)";
+const char * const gnupgext_version = "SHA1 ($Revision: 1.12.2.7 $)";
 
 static struct {
     int class;
@@ -398,7 +398,11 @@ gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
 	    ret = &func_table[i].value;
 	    break;
 	  default:
+#ifndef __riscos__
 	    ret = func_table[i].func;
+#else /* __riscos__ */
+	    ret = (void *) func_table[i].func;
+#endif /* __riscos__ */
 	    break;
 	}
 	i++;
