@@ -1,5 +1,5 @@
 /* delkey.c - delete keys
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -52,27 +52,29 @@ do_delete_key( const char *username, int secret, int *r_sec_avail )
     int rc = 0;
     KBNODE keyblock = NULL;
     KBNODE node;
-    KBPOS kbpos;
+    KEYDB_HANDLE hd = keydb_new (secret);
     PKT_public_key *pk = NULL;
     PKT_secret_key *sk = NULL;
     u32 keyid[2];
     int okay=0;
     int yes;
+    KEYDB_SEARCH_DESC desc;
 
     *r_sec_avail = 0;
+
     /* search the userid */
-    rc = secret? find_secret_keyblock_byname( &kbpos, username )
-	       : find_keyblock_byname( &kbpos, username );
-    if( rc ) {
-	log_error(_("%s: user not found\n"), username );
+    classify_user_id (username, &desc);
+    rc = desc.mode? keydb_search (hd, &desc, 1):G10ERR_INV_USER_ID;
+    if (rc) {
+	log_error (_("key `%s' not found: %s\n"), username, g10_errstr (rc));
 	write_status_text( STATUS_DELETE_PROBLEM, "1" );
 	goto leave;
     }
 
     /* read the keyblock */
-    rc = read_keyblock( &kbpos, &keyblock );
-    if( rc ) {
-	log_error("%s: read problem: %s\n", username, g10_errstr(rc) );
+    rc = keydb_get_keyblock (hd, &keyblock );
+    if (rc) {
+	log_error (_("error reading keyblock: %s\n"), g10_errstr(rc) );
 	goto leave;
     }
 
@@ -149,15 +151,25 @@ do_delete_key( const char *username, int secret, int *r_sec_avail )
 
 
     if( okay ) {
-	rc = delete_keyblock( &kbpos );
-	if( rc ) {
-	    log_error("delete_keyblock failed: %s\n", g10_errstr(rc) );
+	rc = keydb_delete_keyblock (hd);
+	if (rc) {
+	    log_error (_("deleting keyblock failed: %s\n"), g10_errstr(rc) );
 	    goto leave;
 	}
+
+	/* Note that the ownertrust being cleared will trigger a
+           revalidation_mark().  This makes sense - only deleting keys
+           that have ownertrust set should trigger this. */
+
+        if (!secret && pk && clear_ownertrust (pk)) {
+          if (opt.verbose)
+            log_info (_("ownertrust information cleared\n"));
+        }
     }
 
   leave:
-    release_kbnode( keyblock );
+    keydb_release (hd);
+    release_kbnode (keyblock);
     return rc;
 }
 
@@ -165,24 +177,33 @@ do_delete_key( const char *username, int secret, int *r_sec_avail )
  * Delete a public or secret key from a keyring.
  */
 int
-delete_key( const char *username, int secret, int allow_both )
+delete_keys( STRLIST names, int secret, int allow_both )
 {
     int rc, avail;
 
-    rc = do_delete_key (username, secret, &avail );
-    if ( rc && avail ) { 
-        if ( allow_both ) {
-            rc = do_delete_key (username, 1, &avail );
-            if ( !rc )
-                rc = do_delete_key (username, 0, &avail );
-        }
-        else {
-            log_error(_(
-                "there is a secret key for this public key!\n"));
-            log_info(_(
-                "use option \"--delete-secret-key\" to delete it first.\n"));
-            write_status_text( STATUS_DELETE_PROBLEM, "2" );
-        }
+    for(;names;names=names->next) {
+       rc = do_delete_key (names->d, secret, &avail );
+       if ( rc && avail ) { 
+	 if ( allow_both ) {
+	   rc = do_delete_key (names->d, 1, &avail );
+	   if ( !rc )
+	     rc = do_delete_key (names->d, 0, &avail );
+	 }
+	 else {
+	   log_error(_(
+	      "there is a secret key for public key \"%s\"!\n"),names->d);
+	   log_info(_(
+	      "use option \"--delete-secret-keys\" to delete it first.\n"));
+	   write_status_text( STATUS_DELETE_PROBLEM, "2" );
+	   return rc;
+	 }
+       }
+
+       if(rc) {
+	 log_error("%s: delete key failed: %s\n", names->d, g10_errstr(rc) );
+	 return rc;
+       }
     }
-    return rc;
+
+    return 0;
 }
