@@ -1,5 +1,5 @@
 /* export.c
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -83,8 +83,10 @@ do_export( STRLIST users, int secret, int onlyrfc )
     IOBUF out = NULL;
     int any, rc;
     armor_filter_context_t afx;
+    compress_filter_context_t zfx;
 
     memset( &afx, 0, sizeof afx);
+    memset( &zfx, 0, sizeof zfx);
 
     rc = open_outfile( NULL, 0, &out );
     if( rc )
@@ -94,6 +96,8 @@ do_export( STRLIST users, int secret, int onlyrfc )
 	afx.what = secret?5:1;
 	iobuf_push_filter( out, armor_filter, &afx );
     }
+    if( opt.compress_keys && opt.compress )
+	iobuf_push_filter( out, compress_filter, &zfx );
     rc = do_export_stream( out, users, secret, onlyrfc, &any );
 
     if( rc || !any )
@@ -108,50 +112,48 @@ static int
 do_export_stream( IOBUF out, STRLIST users, int secret, int onlyrfc, int *any )
 {
     int rc = 0;
-    compress_filter_context_t zfx;
     PACKET pkt;
     KBNODE keyblock = NULL;
     KBNODE kbctx, node;
+    int ndesc;
+    KEYDB_SEARCH_DESC *desc = NULL;
     KEYDB_HANDLE kdbhd;
     STRLIST sl;
-    int all = !users;
-    int all_first = 1;
 
     *any = 0;
-    memset( &zfx, 0, sizeof zfx);
     init_packet( &pkt );
-
     kdbhd = keydb_new (secret);
 
-    if( opt.compress_keys && opt.compress )
-	iobuf_push_filter( out, compress_filter, &zfx );
-
-    /* use the correct sequence. strlist_last,prev do work correctly with
-     * NULL pointers :-) */
-    for( sl=strlist_last(users); sl || all ; sl=strlist_prev( users, sl )) {
-	if( all ) { /* get the next user */
-	    rc = all_first ? keydb_search_first (kdbhd)
-                           : keydb_search_next (kdbhd);
-            all_first = 0;
-	    if( rc == -1 )  /* EOF */
-		break;
-	    if( rc ) {
-		log_error ("error searching key: %s\n", g10_errstr(rc));
-		break;
-	    }
-	}
-	else {
-            KEYDB_SEARCH_DESC desc;
-
-            classify_user_id (sl->d, &desc);
-            rc = desc.mode? keydb_search (kdbhd, &desc, 1):G10ERR_INV_USER_ID;
-	    if( rc ) {
+    if (!users) {
+        ndesc = 1;
+        desc = m_alloc_clear ( ndesc * sizeof *desc);
+        desc[0].mode = KEYDB_SEARCH_MODE_FIRST;
+    }
+    else {
+        for (ndesc=0, sl=users; sl; sl = sl->next, ndesc++) 
+            ;
+        desc = m_alloc ( ndesc * sizeof *desc);
+        
+        for (ndesc=0, sl=users; sl; sl = sl->next) {
+	    if (classify_user_id (sl->d, desc+ndesc))
+                ndesc++;
+            else
                 log_error (_("key `%s' not found: %s\n"),
-                           sl->d, g10_errstr (rc));
-		rc = 0;
-		continue;
-	    }
-	}
+                           sl->d, g10_errstr (G10ERR_INV_USER_ID));
+        }
+
+        /* it would be nice to see which of the given users did
+           actually match one in the keyring.  To implement this we
+           need to have a found flag for each entry in desc and to set
+           this we must check all those entries after a match to mark
+           all matched one - currently we stop at the first match.  To
+           do this we need an extra flag to enable this feature so */
+    }
+
+
+    while (!(rc = keydb_search (kdbhd, desc, ndesc))) {
+	if (!users) 
+            desc[0].mode = KEYDB_SEARCH_MODE_NEXT;
 
         /* read the keyblock */
         rc = keydb_get_keyblock (kdbhd, &keyblock );
@@ -180,23 +182,44 @@ do_export_stream( IOBUF out, STRLIST users, int secret, int onlyrfc, int *any )
 	    continue;
 	}
 
+	/* no v3 keys with GNU mode 1001 */
+	if( secret == 2 && node->pkt->pkt.secret_key->version == 3 )
+	{
+	    log_info(_("key %08lX: PGP 2.x style key - skipped\n"),
+		  (ulong)keyid_from_sk( node->pkt->pkt.secret_key, NULL) );
+	    continue;
+	}
+
 	/* and write it */
 	for( kbctx=NULL; (node = walk_kbnode( keyblock, &kbctx, 0 )); ) {
 	    /* don't export any comment packets but those in the
 	     * secret keyring */
 	    if( !secret && node->pkt->pkttype == PKT_COMMENT )
 		continue;
-	    /* do not export packets which are marked as not exportable */
-	    if( node->pkt->pkttype == PKT_SIGNATURE ) {
-		const char *p;
-		p = parse_sig_subpkt2( node->pkt->pkt.signature,
-				       SIGSUBPKT_EXPORTABLE, NULL );
-		if( p && !*p )
-		    continue; /* not exportable */
+            /* make sure that ring_trust packets never get exported */
+            if (node->pkt->pkttype == PKT_RING_TRUST)
+              continue;
 
-                /* delete our verification cache */
-                delete_sig_subpkt (node->pkt->pkt.signature->unhashed,
-                                   SIGSUBPKT_PRIV_VERIFY_CACHE);
+	    if( node->pkt->pkttype == PKT_SIGNATURE ) {
+	      /* do not export packets which are marked as not exportable */
+	      if( !node->pkt->pkt.signature->flags.exportable )
+		continue; /* not exportable */
+
+	      /* do not export packets with a "sensitive" revocation
+                 key.  This will need revisiting when we start
+                 supporting creating revocation keys and not just
+                 reading them. */
+	      if( node->pkt->pkt.signature->revkey ) {
+		int i;
+
+		for(i=0;i<node->pkt->pkt.signature->numrevkeys;i++)
+		  if(node->pkt->pkt.signature->revkey[i]->class & 0x40)
+		    continue;
+	      }
+
+	      /* delete our verification cache */
+	      delete_sig_subpkt (node->pkt->pkt.signature->unhashed,
+				 SIGSUBPKT_PRIV_VERIFY_CACHE);
 	    }
 
 	    if( secret == 2 && node->pkt->pkttype == PKT_SECRET_KEY ) {
@@ -225,6 +248,7 @@ do_export_stream( IOBUF out, STRLIST users, int secret, int onlyrfc, int *any )
 	rc = 0;
 
   leave:
+    m_free(desc);
     keydb_release (kdbhd);
     release_kbnode( keyblock );
     if( !*any )
