@@ -1,5 +1,5 @@
 /* free-packet.c - cleanup stuff for packets
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -59,6 +59,8 @@ free_seckey_enc( PKT_signature *sig )
 	mpi_free(sig->data[0]);
     for(i=0; i < n; i++ )
 	mpi_free( sig->data[i] );
+
+    m_free(sig->revkey);
     m_free(sig->hashed);
     m_free(sig->unhashed);
     m_free(sig);
@@ -89,6 +91,11 @@ release_public_key_parts( PKT_public_key *pk )
         free_user_id (pk->user_id);
         pk->user_id = NULL;
     }
+    if (pk->revkey) {
+        m_free(pk->revkey);
+	pk->revkey=NULL;
+	pk->numrevkeys=0;
+    }
 }
 
 
@@ -157,6 +164,8 @@ copy_public_key ( PKT_public_key *d, PKT_public_key *s)
 	for(i=0; i < n; i++ )
 	    d->pkey[i] = mpi_copy( s->pkey[i] );
     }
+    d->revkey=m_alloc(sizeof(struct revocation_key)*s->numrevkeys);
+    memcpy(d->revkey,s->revkey,sizeof(struct revocation_key)*s->numrevkeys);
     return d;
 }
 
@@ -261,6 +270,17 @@ free_comment( PKT_comment *rem )
     m_free(rem);
 }
 
+void
+free_attributes(PKT_user_id *uid)
+{
+  m_free(uid->attribs);
+  m_free(uid->attrib_data);
+
+  uid->attribs=NULL;
+  uid->attrib_data=NULL;
+  uid->attrib_len=0;
+}
+
 void
 free_user_id (PKT_user_id *uid)
 {
@@ -268,8 +288,8 @@ free_user_id (PKT_user_id *uid)
     if (--uid->ref)
         return;
 
-    if (uid->photo)
-	m_free (uid->photo);
+    free_attributes(uid);
+
     if (uid->prefs)
         m_free (uid->prefs);
     m_free (uid);
@@ -370,6 +390,7 @@ free_packet( PACKET *pkt )
 	free_compressed( pkt->pkt.compressed);
 	break;
       case PKT_ENCRYPTED:
+      case PKT_ENCRYPTED_MDC:
 	free_encrypted( pkt->pkt.encrypted );
 	break;
       case PKT_PLAINTEXT:
