@@ -1,5 +1,5 @@
 /* gpgv.c - The GnuPG signature verify utility
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -28,8 +28,11 @@
 #ifdef HAVE_DOSISH_SYSTEM
   #include <fcntl.h> /* for setmode() */
 #endif
+#ifdef __riscos__
+#include <unixlib/local.h>
+#endif /* __riscos__ */
 
-
+#define INCLUDED_BY_MAIN_MODULE 1
 #include "packet.h"
 #include "iobuf.h"
 #include "memory.h"
@@ -45,7 +48,6 @@
 #include "i18n.h"
 #include "status.h"
 #include "g10defs.h"
-#include "hkp.h"
 
 
 enum cmd_and_opt_values { aNull = 0,
@@ -136,17 +138,20 @@ main( int argc, char **argv )
     STRLIST sl;
     STRLIST nrings=NULL;
     unsigned configlineno;
+  #ifdef __riscos__
+    __riscosify_control = __RISCOSIFY_NO_PROCESS;
+  #endif /* __riscos__ */
 
     log_set_name("gpgv");
     init_signals();
     i18n_init();
     opt.command_fd = -1; /* no command fd */
     opt.pgp2_workarounds = 1;
-    opt.auto_key_retrieve = 1;
+    opt.keyserver_options.auto_key_retrieve = 1;
     opt.always_trust = 1;
     opt.batch = 1;
 
-  #ifdef __MINGW32__
+  #if defined (__MINGW32__) || defined (__CYGWIN32__)
     opt.homedir = read_w32_registry_string( NULL, "Software\\GNU\\GnuPG", "HomeDir" );
   #else
     opt.homedir = getenv("GNUPGHOME");
@@ -183,9 +188,9 @@ main( int argc, char **argv )
 	set_packet_list_mode(1);
 
     if( !nrings )  /* no keyring given: use default one */
-        add_keyblock_resource("trustedkeys.gpg", 0, 0);
+        keydb_add_resource ("trustedkeys" EXTSEP_S "gpg", 0, 0);
     for(sl = nrings; sl; sl = sl->next )
-        add_keyblock_resource( sl->d, 0, 0 );
+        keydb_add_resource (sl->d, 0, 0 );
     
     FREE_STRLIST(nrings);
     
@@ -223,21 +228,14 @@ check_signatures_trust( PKT_signature *sig )
  * instead
  */
 int
-keyid_from_lid( ulong lid, u32 *keyid )
-{
-    return G10ERR_TRUSTDB;
-}
-
-/* Stub: */
-int
-query_trust_info( PKT_public_key *pk, const byte *namehash )
+get_validity_info (PKT_public_key *pk, const byte *namehash )
 {
     return '?';
 }
 
 /* Stub: */
 int
-get_ownertrust_info( ulong lid )
+get_ownertrust_info (PKT_public_key *pk)
 {
     return '?';
 }
@@ -248,7 +246,7 @@ get_ownertrust_info( ulong lid )
  * get them from a keyserver
  */
 int
-hkp_ask_import( u32 *keyid )
+keyserver_import_keyid( u32 *keyid, void *dummy )
 {
     return -1;
 }
@@ -297,11 +295,14 @@ check_secret_key( PKT_secret_key *sk, int n )
  */
 DEK *
 passphrase_to_dek( u32 *keyid, int pubkey_algo,
-		   int cipher_algo, STRING2KEY *s2k, int mode )
+		   int cipher_algo, STRING2KEY *s2k, int mode,
+                   const char *tmp)
 {
     return NULL;
 }
 
+/* Stub to avoid linking to photoid.c */
+void show_photo(const struct user_attribute *attr,PKT_public_key *pk) {}
 
 /* Stubs to void linking to ../cipher/cipher.c */
 int string_to_cipher_algo( const char *string ) { return 0; }
@@ -359,15 +360,3 @@ DOTLOCK create_dotlock( const char *file_to_lock ) { return NULL; }
 int make_dotlock( DOTLOCK h, long timeout ) { return 0;}
 int release_dotlock( DOTLOCK h ) {return 0;}
 void remove_lockfiles(void) {}
-
-
-
-
-
-
-
-
-
-
-
-
