@@ -1,5 +1,5 @@
 /* keygen.c - generate a key pair
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -32,6 +32,7 @@
 #include "ttyio.h"
 #include "options.h"
 #include "keydb.h"
+#include "trustdb.h"
 #include "status.h"
 #include "i18n.h"
 
@@ -48,6 +49,7 @@ enum para_name {
   pNAMEREAL,
   pNAMEEMAIL,
   pNAMECOMMENT,
+  pPREFERENCES,
   pUSERID,
   pEXPIREDATE,
   pKEYEXPIRE, /* in n seconds */
@@ -124,17 +126,17 @@ write_uid( KBNODE root, const char *s )
 }
 
 static void
-do_add_key_flags (PKT_signature *sig, unsigned int usage)
+do_add_key_flags (PKT_signature *sig, unsigned int use)
 {
     byte buf[1];
 
-    if (!usage) 
+    if (!use) 
         return;
 
     buf[0] = 0;
-    if (usage & PUBKEY_USAGE_SIG)
+    if (use & PUBKEY_USAGE_SIG)
         buf[0] |= 0x01 | 0x02;
-    if (usage & PUBKEY_USAGE_ENC)
+    if (use & PUBKEY_USAGE_ENC)
         buf[0] |= 0x04 | 0x08;
     build_sig_subpkt (sig, SIGSUBPKT_KEY_FLAGS, buf, 1);
 }
@@ -174,8 +176,10 @@ set_one_pref (ulong val, int type, int (*cf)(int), byte *buf, int *nbuf)
 {
     int i;
 
-    if (!val || val > 127 || cf (val)) {
+    if (cf (val)) {
         log_info (_("preference %c%lu is not valid\n"), type, val);
+	if(type=='S' && val==CIPHER_ALGO_IDEA)
+	  idea_cipher_warn(1);
         return -1;
     }
     for (i=0; i < *nbuf; i++ ) {
@@ -214,8 +218,20 @@ keygen_set_std_prefs (const char *string)
     const char *s, *s2;
     int rc = 0;
 
-    if (!string || !ascii_strcasecmp (string, "default"))
-        string = "S7 S10 S3 S4 H3 H2 Z2 Z1";
+    if (!string || !ascii_strcasecmp (string, "default")) {
+      if ( !check_cipher_algo(CIPHER_ALGO_IDEA) )
+        string = "S7 S3 S2 S1 H2 H3 Z2 Z1";
+      else
+        string = "S7 S3 S2 H2 H3 Z2 Z1";
+
+      /* If we have it, IDEA goes *after* 3DES so it won't be used
+         unless we're encrypting along with a V3 key.  Ideally, we
+         would only put the S1 preference in if the key was RSA and
+         <=2048 bits, as that is what won't break PGP2, but that is
+         difficult with the current code, and not really worth
+         checking as a non-RSA <=2048 bit key wouldn't be usable by
+         PGP2 anyway -dms */
+    }
     else if (!ascii_strcasecmp (string, "none"))
         string = "";
 
@@ -238,7 +254,7 @@ keygen_set_std_prefs (const char *string)
         else if (isspace (*s))
             s2 = s+1;
         else {
-            log_info (_("invalid character in string\n"));
+            log_info (_("invalid character in preference string\n"));
             return -1;
         }
     }
@@ -292,7 +308,7 @@ add_feature_mdc (PKT_signature *sig)
         return; /* already set */
     if (!s || !n) { /* create a new one */
         n = 1;
-        buf = m_alloc (n);
+        buf = m_alloc_clear (n);
     }
     else {
         buf = m_alloc (n);
@@ -313,15 +329,26 @@ keygen_upd_std_prefs( PKT_signature *sig, void *opaque )
     if (nsym_prefs) 
         build_sig_subpkt (sig, SIGSUBPKT_PREF_SYM, sym_prefs, nsym_prefs);
     else
+      {
         delete_sig_subpkt (sig->hashed, SIGSUBPKT_PREF_SYM);
+        delete_sig_subpkt (sig->unhashed, SIGSUBPKT_PREF_SYM);
+      }
+
     if (nhash_prefs)
         build_sig_subpkt (sig, SIGSUBPKT_PREF_HASH, hash_prefs, nhash_prefs);
     else
-        delete_sig_subpkt (sig->hashed, SIGSUBPKT_PREF_HASH);
+      {
+	delete_sig_subpkt (sig->hashed, SIGSUBPKT_PREF_HASH);
+	delete_sig_subpkt (sig->unhashed, SIGSUBPKT_PREF_HASH);
+      }
+
     if (nzip_prefs)
         build_sig_subpkt (sig, SIGSUBPKT_PREF_COMPR, zip_prefs, nzip_prefs);
     else
+      {
         delete_sig_subpkt (sig->hashed, SIGSUBPKT_PREF_COMPR);
+        delete_sig_subpkt (sig->unhashed, SIGSUBPKT_PREF_COMPR);
+      }
 
     /* Make sure that the MDC feature flag is set */
     add_feature_mdc (sig);
@@ -333,6 +360,7 @@ keygen_upd_std_prefs( PKT_signature *sig, void *opaque )
 /****************
  * Add preference to the self signature packet.
  * This is only called for packets with version > 3.
+
  */
 int
 keygen_add_std_prefs( PKT_signature *sig, void *opaque )
@@ -358,7 +386,7 @@ keygen_add_std_prefs( PKT_signature *sig, void *opaque )
 
 static int
 write_selfsig( KBNODE root, KBNODE pub_root, PKT_secret_key *sk,
-               unsigned int usage )
+               unsigned int use )
 {
     PACKET *pkt;
     PKT_signature *sig;
@@ -380,13 +408,13 @@ write_selfsig( KBNODE root, KBNODE pub_root, PKT_secret_key *sk,
     if( !node )
 	BUG();
     pk = node->pkt->pkt.public_key;
-    pk->pubkey_usage = usage;
+    pk->pubkey_usage = use;
     /* we have to cache the key, so that the verification of the signature
      * creation is able to retrieve the public key */
     cache_public_key (pk);
 
     /* and make the signature */
-    rc = make_keysig_packet( &sig, pk, uid, NULL, sk, 0x13, 0, 0,
+    rc = make_keysig_packet( &sig, pk, uid, NULL, sk, 0x13, 0, 0, 0, 0,
         		     keygen_add_std_prefs, pk );
     if( rc ) {
 	log_error("make_keysig_packet failed: %s\n", g10_errstr(rc) );
@@ -402,7 +430,7 @@ write_selfsig( KBNODE root, KBNODE pub_root, PKT_secret_key *sk,
 
 static int
 write_keybinding( KBNODE root, KBNODE pub_root, PKT_secret_key *sk,
-                  unsigned int usage )
+                  unsigned int use )
 {
     PACKET *pkt;
     PKT_signature *sig;
@@ -433,9 +461,9 @@ write_keybinding( KBNODE root, KBNODE pub_root, PKT_secret_key *sk,
 	BUG();
 
     /* and make the signature */
-    oduap.usage = usage;
+    oduap.usage = use;
     oduap.pk = subpk;
-    rc = make_keysig_packet( &sig, pk, NULL, subpk, sk, 0x18, 0, 0,
+    rc = make_keysig_packet( &sig, pk, NULL, subpk, sk, 0x18, 0, 0, 0, 0,
         		     keygen_add_key_flags_and_expire, &oduap );
     if( rc ) {
 	log_error("make_keysig_packet failed: %s\n", g10_errstr(rc) );
@@ -804,7 +832,7 @@ ask_keysize( int algo )
     char *answer;
     unsigned nbits;
 
-    if (algo != PUBKEY_ALGO_DSA) {
+    if (algo != PUBKEY_ALGO_DSA && algo != PUBKEY_ALGO_RSA) {
         tty_printf (_("About to generate a new %s keypair.\n"
                       "              minimum keysize is  768 bits\n"
                       "              default keysize is 1024 bits\n"
@@ -820,11 +848,12 @@ ask_keysize( int algo )
 	m_free(answer);
 	if( algo == PUBKEY_ALGO_DSA && (nbits < 512 || nbits > 1024) )
 	    tty_printf(_("DSA only allows keysizes from 512 to 1024\n"));
-	else if( nbits < 768 )
-	    tty_printf(_("keysize too small; 768 is smallest value allowed.\n"));
 	else if( algo == PUBKEY_ALGO_RSA && nbits < 1024 )
 	    tty_printf(_("keysize too small;"
 			 " 1024 is smallest value allowed for RSA.\n"));
+	else if( nbits < 768 )
+	    tty_printf(_("keysize too small;"
+			 " 768 is smallest value allowed.\n"));
 	else if( nbits > 4096 ) {
 	    /* It is ridiculous and an annoyance to use larger key sizes!
 	     * GnuPG can handle much larger sizes; but it takes an eternity
@@ -849,11 +878,6 @@ ask_keysize( int algo )
 		break;
 	    }
 	}
-	else if( nbits > 1536 && !cpr_enabled() && algo != PUBKEY_ALGO_RSA ) {
-	    if( cpr_get_answer_is_yes("keygen.size.large.okay",_(
-		    "Do you really need such a large keysize? ")) )
-		break;
-	}
 	else
 	    break;
     }
@@ -904,20 +928,38 @@ parse_expire_string( const char *string )
     return valid_days;
 }
 
-
-static u32
-ask_expire_interval(void)
+/* object == 0 for a key, and 1 for a sig */
+u32
+ask_expire_interval(int object)
 {
     char *answer;
     int valid_days=0;
     u32 interval = 0;
 
-    tty_printf(_("Please specify how long the key should be valid.\n"
-		 "         0 = key does not expire\n"
-		 "      <n>  = key expires in n days\n"
-		 "      <n>w = key expires in n weeks\n"
-		 "      <n>m = key expires in n months\n"
-		 "      <n>y = key expires in n years\n"));
+    switch(object)
+      {
+      case 0:
+	tty_printf(_("Please specify how long the key should be valid.\n"
+		     "         0 = key does not expire\n"
+		     "      <n>  = key expires in n days\n"
+		     "      <n>w = key expires in n weeks\n"
+		     "      <n>m = key expires in n months\n"
+		     "      <n>y = key expires in n years\n"));
+	break;
+
+      case 1:
+	tty_printf(_("Please specify how long the signature should be valid.\n"
+		     "         0 = signature does not expire\n"
+		     "      <n>  = signature expires in n days\n"
+		     "      <n>w = signature expires in n weeks\n"
+		     "      <n>m = signature expires in n months\n"
+		     "      <n>y = signature expires in n years\n"));
+	break;
+
+      default:
+	BUG();
+      }
+
     /* Note: The elgamal subkey for DSA has no expiration date because
      * it must be signed with the DSA key and this one has the expiration
      * date */
@@ -927,7 +969,10 @@ ask_expire_interval(void)
 	u32 curtime=make_timestamp();
 
 	m_free(answer);
-	answer = cpr_get("keygen.valid",_("Key is valid for? (0) "));
+	if(object==0)
+	  answer = cpr_get("keygen.valid",_("Key is valid for? (0) "));
+	else
+	  answer = cpr_get("siggen.valid",_("Signature is valid for? (0) "));
 	cpr_kill_prompt();
 	trim_spaces(answer);
 	valid_days = parse_expire_string( answer );
@@ -937,13 +982,15 @@ ask_expire_interval(void)
 	}
 
 	if( !valid_days ) {
-	    tty_printf(_("Key does not expire at all\n"));
+	    tty_printf(_("%s does not expire at all\n"),
+		       object==0?"Key":"Signature");
 	    interval = 0;
 	}
 	else {
 	    interval = valid_days * 86400L;
 	    /* print the date when the key expires */
-	    tty_printf(_("Key expires at %s\n"),
+	    tty_printf(_("%s expires at %s\n"),
+		        object==0?"Key":"Signature",
 			asctimestamp((ulong)(curtime + interval) ) );
             /* FIXME: This check yields warning on alhas:
                write a configure check and to this check here only for 32 bit machines */
@@ -963,7 +1010,7 @@ ask_expire_interval(void)
 u32
 ask_expiredate()
 {
-    u32 x = ask_expire_interval();
+    u32 x = ask_expire_interval(0);
     return x? make_timestamp() + x : 0;
 }
 
@@ -1158,6 +1205,7 @@ ask_passphrase( STRING2KEY **ret_s2k )
 {
     DEK *dek = NULL;
     STRING2KEY *s2k;
+    const char *errtext = NULL;
 
     tty_printf(_("You need a Passphrase to protect your secret key.\n\n") );
 
@@ -1165,9 +1213,10 @@ ask_passphrase( STRING2KEY **ret_s2k )
     for(;;) {
 	s2k->mode = opt.s2k_mode;
 	s2k->hash_algo = opt.s2k_digest_algo;
-	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2 );
+	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k,2,errtext);
 	if( !dek ) {
-	    tty_printf(_("passphrase not correctly repeated; try again.\n"));
+	    errtext = _("passphrase not correctly repeated; try again");
+	    tty_printf(_("%s.\n"), errtext);
 	}
 	else if( !dek->keylen ) {
 	    m_free(dek); dek = NULL;
@@ -1239,6 +1288,7 @@ generate_user_id()
     uid = m_alloc_clear( sizeof *uid + n - 1 );
     uid->len = n;
     strcpy(uid->name, p);
+    uid->ref = 1;
     return uid;
 }
 
@@ -1296,26 +1346,26 @@ parse_parameter_usage (const char *fname,
 {
     struct para_data_s *r = get_parameter( para, key );
     char *p, *pn;
-    unsigned int usage;
+    unsigned int use;
 
     if( !r )
 	return 0; /* none (this is an optional parameter)*/
     
-    usage = 0;
+    use = 0;
     pn = r->u.value;
     while ( (p = strsep (&pn, " \t,")) ) {
         if ( !*p)
             ;
         else if ( !ascii_strcasecmp (p, "sign") )
-            usage |= PUBKEY_USAGE_SIG;
+            use |= PUBKEY_USAGE_SIG;
         else if ( !ascii_strcasecmp (p, "encrypt") )
-            usage |= PUBKEY_USAGE_ENC;
+            use |= PUBKEY_USAGE_ENC;
         else {
             log_error("%s:%d: invalid usage list\n", fname, r->lnr );
             return -1; /* error */
         }
     }
-    r->u.usage = usage;
+    r->u.usage = use;
     return 0;
 }
 
@@ -1409,6 +1459,9 @@ proc_parameter_file( struct para_data_s *para, const char *fname,
 	}
     }
 
+    /* Set preferences, if any. */
+    keygen_set_std_prefs(get_parameter_value( para, pPREFERENCES ));
+
     /* make DEK and S2K from the Passphrase */
     r = get_parameter( para, pPASSPHRASE );
     if( r && *r->u.value ) {
@@ -1422,7 +1475,7 @@ proc_parameter_file( struct para_data_s *para, const char *fname,
 	s2k->mode = opt.s2k_mode;
 	s2k->hash_algo = opt.s2k_digest_algo;
 	set_next_passphrase( r->u.value );
-	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2 );
+	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2, NULL );
 	set_next_passphrase( NULL );
 	assert( dek );
 	memset( r->u.value, 0, strlen(r->u.value) );
@@ -1489,6 +1542,7 @@ read_parameter_file( const char *fname )
 	{ "Name-Comment",   pNAMECOMMENT },
 	{ "Expire-Date",    pEXPIREDATE },
 	{ "Passphrase",     pPASSPHRASE },
+	{ "Preferences",    pPREFERENCES },
 	{ NULL, 0 }
     };
     FILE *fp;
@@ -1662,7 +1716,7 @@ generate_keypair( const char *fname )
     DEK *dek;
     STRING2KEY *s2k;
     int algo;
-    unsigned int usage;
+    unsigned int use;
     int both = 0;
     u32 expire;
     struct para_data_s *para = NULL;
@@ -1676,7 +1730,7 @@ generate_keypair( const char *fname )
 	return;
     }
 
-    algo = ask_algo( 0, &usage );
+    algo = ask_algo( 0, &use );
     if( !algo ) { /* default: DSA with ElG subkey of the specified size */
 	both = 1;
 	r = m_alloc_clear( sizeof *r + 20 );
@@ -1705,12 +1759,12 @@ generate_keypair( const char *fname )
 	r->next = para;
 	para = r;
 
-        if (usage) {
+        if (use) {
             r = m_alloc_clear( sizeof *r + 20 );
             r->key = pKEYUSAGE;
             sprintf( r->u.value, "%s%s",
-                     (usage & PUBKEY_USAGE_SIG)? "sign ":"",
-                     (usage & PUBKEY_USAGE_ENC)? "encrypt ":"" );
+                     (use & PUBKEY_USAGE_SIG)? "sign ":"",
+                     (use & PUBKEY_USAGE_ENC)? "encrypt ":"" );
             r->next = para;
             para = r;
         }
@@ -1724,7 +1778,7 @@ generate_keypair( const char *fname )
     r->next = para;
     para = r;
 
-    expire = ask_expire_interval();
+    expire = ask_expire_interval(0);
     r = m_alloc_clear( sizeof *r + 20 );
     r->key = pKEYEXPIRE;
     r->u.expire = expire;
@@ -1934,9 +1988,21 @@ do_generate_keypair( struct para_data_s *para,
                 get_parameter_algo(para, pKEYTYPE) == PUBKEY_ALGO_RSA
                 && get_parameter_uint( para, pKEYUSAGE )
                 && !(get_parameter_uint( para,pKEYUSAGE) & PUBKEY_USAGE_ENC);
+            PKT_public_key *pk = find_kbnode (pub_root, 
+                                    PKT_PUBLIC_KEY)->pkt->pkt.public_key;
+            
+            update_ownertrust (pk,
+                               ((get_ownertrust (pk) & ~TRUST_MASK)
+                                | TRUST_ULTIMATE ));
+
+	    if (!opt.batch) {
+                tty_printf(_("public and secret key created and signed.\n") );
+                tty_printf(_("key marked as ultimately trusted.\n") );
+		tty_printf("\n");
+		list_keyblock(pub_root,0,1,NULL);
+            }
+            
 
-	    if( !opt.batch )
-		 tty_printf(_("public and secret key created and signed.\n") );
 	    if( !opt.batch
 		&& ( get_parameter_algo( para, pKEYTYPE ) == PUBKEY_ALGO_DSA
                      || no_enc_rsa )
@@ -1977,7 +2043,7 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
     KBNODE node;
     PKT_secret_key *sk = NULL; /* this is the primary sk */
     int algo;
-    unsigned int usage;
+    unsigned int use;
     u32 expire;
     unsigned nbits;
     char *passphrase = NULL;
@@ -2008,9 +2074,11 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
 	}
     }
 
-    if (sk->version < 4) 
+    if (sk->version < 4) {
         log_info (_("NOTE: creating subkeys for v3 keys "
                     "is not OpenPGP compliant\n"));
+	goto leave;
+    }
 
     /* unprotect to get the passphrase */
     switch( is_secret_key_protected( sk ) ) {
@@ -2031,10 +2099,10 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
 	goto leave;
 
 
-    algo = ask_algo( 1, &usage );
+    algo = ask_algo( 1, &use );
     assert(algo);
     nbits = ask_keysize( algo );
-    expire = ask_expire_interval();
+    expire = ask_expire_interval(0);
     if( !cpr_enabled() && !cpr_get_answer_is_yes("keygen.sub.okay",
 						  _("Really create? ") ) )
 	goto leave;
@@ -2044,15 +2112,15 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
 	s2k->mode = opt.s2k_mode;
 	s2k->hash_algo = opt.s2k_digest_algo;
 	set_next_passphrase( passphrase );
-	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2 );
+	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2, NULL );
     }
 
     rc = do_create( algo, nbits, pub_keyblock, sec_keyblock,
 				      dek, s2k, NULL, expire );
     if( !rc )
-	rc = write_keybinding(pub_keyblock, pub_keyblock, sk, usage);
+	rc = write_keybinding(pub_keyblock, pub_keyblock, sk, use);
     if( !rc )
-	rc = write_keybinding(sec_keyblock, pub_keyblock, sk, usage);
+	rc = write_keybinding(sec_keyblock, pub_keyblock, sk, use);
     if( !rc ) {
 	okay = 1;
         write_status_text (STATUS_KEY_CREATED, "S");
