@@ -1,5 +1,5 @@
 /* keylist.c
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -30,16 +30,22 @@
 #include "errors.h"
 #include "keydb.h"
 #include "memory.h"
+#include "photoid.h"
 #include "util.h"
+#include "ttyio.h"
 #include "trustdb.h"
 #include "main.h"
 #include "i18n.h"
 
 static void list_all(int);
 static void list_one( STRLIST names, int secret);
-static void list_keyblock( KBNODE keyblock, int secret );
-static void fingerprint( PKT_public_key *pk, PKT_secret_key *sk );
 
+struct sig_stats
+{
+  int inv_sigs;
+  int no_key;
+  int oth_err;
+};
 
 /****************
  * List the keys
@@ -63,45 +69,140 @@ secret_key_list( STRLIST list )
 	list_one( list, 1 );
 }
 
+void
+show_policy_url(PKT_signature *sig,int indent)
+{
+  const byte *p;
+  size_t len;
+  int seq=0;
+
+  while((p=enum_sig_subpkt(sig->hashed,SIGSUBPKT_POLICY,&len,&seq)))
+    {
+      int i;
+
+      for(i=0;i<indent;i++)
+	putchar(' ');
+
+      /* This isn't UTF8 as it is a URL(?) */
+      printf(_("Signature policy: "));
+      print_string(stdout,p,len,0);
+      printf("\n");
+    }
+}
+
+void
+show_notation(PKT_signature *sig,int indent)
+{
+  const byte *p;
+  size_t len;
+  int seq=0;
+
+  /* There may be multiple notations in the same sig. */
+
+  while((p=enum_sig_subpkt(sig->hashed,SIGSUBPKT_NOTATION,&len,&seq)))
+    if(len>=8)
+      {
+	int n1,n2,i;
+
+	n1=(p[4]<<8)|p[5];
+	n2=(p[6]<<8)|p[7];
+
+	if(8+n1+n2!=len)
+	  {
+	    log_info(_("WARNING: invalid notation data found\n"));
+	    return;
+	  }
+
+	for(i=0;i<indent;i++)
+	  putchar(' ');
+
+	/* This is UTF8 */
+	printf(_("Signature notation: "));
+	print_utf8_string(stdout,p+8,n1);
+	printf("=");
+
+	if(*p&0x80)
+	  print_utf8_string(stdout,p+8+n1,n2);
+	else
+	  printf("[ %s ]",_("not human readable"));
+
+	printf("\n");
+      }
+  else
+    log_info(_("WARNING: invalid notation data found\n"));
+}
+
+static void
+print_signature_stats(struct sig_stats *s)
+{
+  if( s->inv_sigs == 1 )
+    tty_printf(_("1 bad signature\n") );
+  else if( s->inv_sigs )
+    tty_printf(_("%d bad signatures\n"), s->inv_sigs );
+  if( s->no_key == 1 )
+    tty_printf(_("1 signature not checked due to a missing key\n") );
+  else if( s->no_key )
+    tty_printf(_("%d signatures not checked due to missing keys\n"),s->no_key);
+  if( s->oth_err == 1 )
+    tty_printf(_("1 signature not checked due to an error\n") );
+  else if( s->oth_err )
+    tty_printf(_("%d signatures not checked due to errors\n"), s->oth_err );
+}
 
 static void
 list_all( int secret )
 {
-    KBPOS kbpos;
+    KEYDB_HANDLE hd;
     KBNODE keyblock = NULL;
     int rc=0;
-    int lastresno;
+    const char *lastresname, *resname;
+    struct sig_stats stats;
 
-    rc = enum_keyblocks( secret? 5:0, &kbpos, &keyblock );
+    memset(&stats,0,sizeof(stats));
+
+    hd = keydb_new (secret);
+    if (!hd)
+        rc = G10ERR_GENERAL;
+    else
+        rc = keydb_search_first (hd);
     if( rc ) {
 	if( rc != -1 )
-	    log_error("enum_keyblocks(open) failed: %s\n", g10_errstr(rc) );
+	    log_error("keydb_search_first failed: %s\n", g10_errstr(rc) );
 	goto leave;
     }
 
-    lastresno = -1;
-    while( !(rc = enum_keyblocks( 1, &kbpos, &keyblock )) ) {
-	if( lastresno != kbpos.resno ) {
-	    const char *s = keyblock_resource_name( &kbpos );
+    lastresname = NULL;
+    do {
+        rc = keydb_get_keyblock (hd, &keyblock);
+        if (rc) {
+            log_error ("keydb_get_keyblock failed: %s\n", g10_errstr(rc));
+            goto leave;
+        }
+        resname = keydb_get_resource_name (hd);
+	if (lastresname != resname ) {
 	    int i;
 
-	    lastresno = kbpos.resno;
-	    printf("%s\n", s );
-	    for(i=strlen(s); i; i-- )
+	    printf("%s\n", resname );
+	    for(i=strlen(resname); i; i-- )
 		putchar('-');
 	    putchar('\n');
+            lastresname = resname;
 	}
         merge_keys_and_selfsig( keyblock );
-	list_keyblock( keyblock, secret );
-	release_kbnode( keyblock ); keyblock = NULL;
-    }
-
+	list_keyblock( keyblock, secret, opt.fingerprint,
+		       opt.check_sigs?&stats:NULL);
+	release_kbnode( keyblock ); 
+        keyblock = NULL;
+    } while (!(rc = keydb_search_next (hd)));
     if( rc && rc != -1 )
-	log_error("enum_keyblocks(read) failed: %s\n", g10_errstr(rc));
+	log_error ("keydb_search_next failed: %s\n", g10_errstr(rc));
+
+    if(opt.check_sigs && !opt.with_colons)
+      print_signature_stats(&stats);
 
   leave:
-    enum_keyblocks( 2, &kbpos, &keyblock ); /* close */
-    release_kbnode( keyblock );
+    release_kbnode (keyblock);
+    keydb_release (hd);
 }
 
 
@@ -111,7 +212,22 @@ list_one( STRLIST names, int secret )
     int rc = 0;
     KBNODE keyblock = NULL;
     GETKEY_CTX ctx;
+    const char *resname;
+    char *keyring_str = N_("Keyring");
+    int i;
+    struct sig_stats stats;
 
+    memset(&stats,0,sizeof(stats));
+
+    /* fixme: using the bynames function has the disadvantage that we
+     * don't know wether one of the names given was not found.  OTOH,
+     * this function has the advantage to list the names in the
+     * sequence as defined by the keyDB and does not duplicate
+     * outputs.  A solution could be do test whether all given have
+     * been listed (this needs a way to use the keyDB search
+     * functions) or to have the search function return indicators for
+     * found names.  Yet another way is to use the keydb search
+     * facilities directly. */
     if( secret ) {
 	rc = get_seckey_bynames( &ctx, NULL, names, &keyblock );
 	if( rc ) {
@@ -120,7 +236,14 @@ list_one( STRLIST names, int secret )
 	    return;
 	}
 	do {
-	    list_keyblock( keyblock, 1 );
+	    if (opt.show_keyring) {
+		resname = keydb_get_resource_name (get_ctx_handle(ctx));
+		printf("%s: %s\n", keyring_str, resname);
+		for(i = strlen(resname) + strlen(keyring_str) + 2; i; i-- )
+		    putchar('-');
+		putchar('\n');
+	    }
+	    list_keyblock( keyblock, 1, opt.fingerprint, &stats );
 	    release_kbnode( keyblock );
 	} while( !get_seckey_next( ctx, NULL, &keyblock ) );
 	get_seckey_end( ctx );
@@ -133,11 +256,22 @@ list_one( STRLIST names, int secret )
 	    return;
 	}
 	do {
-	    list_keyblock( keyblock, 0 );
+	    if (opt.show_keyring) {
+		resname = keydb_get_resource_name (get_ctx_handle(ctx));
+		printf("%s: %s\n", keyring_str, resname);
+		for(i = strlen(resname) + strlen(keyring_str) + 2; i; i-- )
+		    putchar('-');
+		putchar('\n');
+	    }
+	    list_keyblock( keyblock, 0, opt.fingerprint,
+			   opt.check_sigs?&stats:NULL );
 	    release_kbnode( keyblock );
 	} while( !get_pubkey_next( ctx, NULL, &keyblock ) );
 	get_pubkey_end( ctx );
     }
+
+    if(opt.check_sigs && !opt.with_colons)
+      print_signature_stats(&stats);
 }
 
 static void
@@ -157,12 +291,12 @@ print_key_data( PKT_public_key *pk, u32 *keyid )
 static void
 print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
 {
-    unsigned int usage = pk? pk->pubkey_usage : sk->pubkey_usage;
+    unsigned int use = pk? pk->pubkey_usage : sk->pubkey_usage;
     
-    if ( usage & PUBKEY_USAGE_ENC ) {
+    if ( use & PUBKEY_USAGE_ENC ) {
         putchar ('e');
     }
-    if ( usage & PUBKEY_USAGE_SIG ) {
+    if ( use & PUBKEY_USAGE_SIG ) {
         putchar ('s');
         putchar ('c');
     }
@@ -202,9 +336,8 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
     putchar(':');
 }
 
-
 static void
-list_keyblock_print ( KBNODE keyblock, int secret )
+list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 {
     int rc = 0;
     KBNODE kbctx;
@@ -213,6 +346,7 @@ list_keyblock_print ( KBNODE keyblock, int secret )
     PKT_secret_key *sk;
     u32 keyid[2];
     int any=0;
+    struct sig_stats *stats=opaque;
 
     /* get the keyid from the keyblock */
     node = find_kbnode( keyblock, secret? PKT_SECRET_KEY : PKT_PUBLIC_KEY );
@@ -243,6 +377,12 @@ list_keyblock_print ( KBNODE keyblock, int secret )
 
     for( kbctx=NULL; (node=walk_kbnode( keyblock, &kbctx, 0)) ; ) {
 	if( node->pkt->pkttype == PKT_USER_ID && !opt.fast_list_mode ) {
+            /* don't list revoked UIDS unless we are in verbose mode and 
+             * signature listing has not been requested */
+            if ( !opt.verbose && !opt.list_sigs
+                 && node->pkt->pkt.user_id->is_revoked )
+                continue; 
+
 	    if( any ) 
                 printf("uid%*s", 28, "");
 
@@ -252,12 +392,16 @@ list_keyblock_print ( KBNODE keyblock, int secret )
                                node->pkt->pkt.user_id->len );
 	    putchar('\n');
 	    if( !any ) {
-		if( opt.fingerprint )
-		    fingerprint( pk, sk );
+		if( fpr )
+		    print_fingerprint( pk, sk, 0 );
 		if( opt.with_key_data )
 		    print_key_data( pk, keyid );
 		any = 1;
 	    }
+
+	    if(opt.show_photos && node->pkt->pkt.user_id->attribs!=NULL &&
+	       node->pkt->pkt.user_id->attribs->type==ATTRIB_JPEG)
+	      show_photo(node->pkt->pkt.user_id->attribs,pk);
 	}
 	else if( node->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
 	    u32 keyid2[2];
@@ -265,8 +409,8 @@ list_keyblock_print ( KBNODE keyblock, int secret )
 
 	    if( !any ) {
 		putchar('\n');
-		if( opt.fingerprint )
-		    fingerprint( pk, sk ); /* of the main key */
+		if( fpr )
+		    print_fingerprint( pk, sk, 0 ); /* of the main key */
 		any = 1;
 	    }
 
@@ -279,8 +423,8 @@ list_keyblock_print ( KBNODE keyblock, int secret )
                 printf(_(" [expires: %s]"), expirestr_from_pk( pk2 ) );
             }
             putchar('\n');
-	    if( opt.fingerprint > 1 )
-		fingerprint( pk2, NULL );
+	    if( fpr > 1 )
+		print_fingerprint( pk2, NULL, 0 );
 	    if( opt.with_key_data )
 		print_key_data( pk2, keyid2 );
 	}
@@ -290,8 +434,8 @@ list_keyblock_print ( KBNODE keyblock, int secret )
 
 	    if( !any ) {
 		putchar('\n');
-		if( opt.fingerprint )
-		    fingerprint( pk, sk ); /* of the main key */
+		if( fpr )
+		    print_fingerprint( pk, sk, 0 ); /* of the main key */
 		any = 1;
 	    }
 
@@ -300,16 +444,36 @@ list_keyblock_print ( KBNODE keyblock, int secret )
 					   pubkey_letter( sk2->pubkey_algo ),
 					   (ulong)keyid2[1],
 					   datestr_from_sk( sk2 ) );
-	    if( opt.fingerprint > 1 )
-		fingerprint( NULL, sk2 );
+	    if( fpr > 1 )
+		print_fingerprint( NULL, sk2, 0 );
 	}
 	else if( opt.list_sigs && node->pkt->pkttype == PKT_SIGNATURE ) {
 	    PKT_signature *sig = node->pkt->pkt.signature;
 	    int sigrc;
             char *sigstr;
 
+	    if( stats ) {
+                /*fflush(stdout);*/
+		rc = check_key_signature( keyblock, node, NULL );
+		switch( rc ) {
+		 case 0:		 sigrc = '!'; break;
+		 case G10ERR_BAD_SIGN:   stats->inv_sigs++; sigrc = '-'; break;
+		 case G10ERR_NO_PUBKEY: 
+		 case G10ERR_UNU_PUBKEY: stats->no_key++; continue;
+		 default:		 stats->oth_err++; sigrc = '%'; break;
+		}
+	    }
+	    else {
+		rc = 0;
+		sigrc = ' ';
+	    }
+
 	    if( !any ) { /* no user id, (maybe a revocation follows)*/
-		if( sig->sig_class == 0x20 )
+	      /* Check if the pk is really revoked - there could be a
+                 0x20 sig packet there even if we are not revoked
+                 (say, if a revocation key issued the packet, but the
+                 revocation key isn't present to verify it.) */
+		if( sig->sig_class == 0x20 && pk->is_revoked )
 		    puts("[revoked]");
 		else if( sig->sig_class == 0x18 )
 		    puts("[key binding]");
@@ -317,8 +481,8 @@ list_keyblock_print ( KBNODE keyblock, int secret )
 		    puts("[subkey revoked]");
 		else
 		    putchar('\n');
-		if( opt.fingerprint )
-		    fingerprint( pk, sk );
+		if( fpr )
+		    print_fingerprint( pk, sk, 0 );
 		any=1;
 	    }
 
@@ -329,29 +493,24 @@ list_keyblock_print ( KBNODE keyblock, int secret )
 	       sigstr = "sig";
 	    else if( sig->sig_class == 0x18 )
 	       sigstr = "sig";
+	    else if( sig->sig_class == 0x1F )
+	       sigstr = "sig";
 	    else {
                 printf("sig                             "
 		       "[unexpected signature class 0x%02x]\n",sig->sig_class );
 		continue;
 	    }
-	    if( opt.check_sigs ) {
-		fflush(stdout);
-		rc = check_key_signature( keyblock, node, NULL );
-		switch( rc ) {
-		  case 0:		   sigrc = '!'; break;
-		  case G10ERR_BAD_SIGN:    sigrc = '-'; break;
-		  case G10ERR_NO_PUBKEY: 
-		  case G10ERR_UNU_PUBKEY:  sigrc = '?'; break;
-		  default:		   sigrc = '%'; break;
-		}
-	    }
-	    else {
-		rc = 0;
-		sigrc = ' ';
-	    }
+
             fputs( sigstr, stdout );
-            printf("%c       %08lX %s  ",
-		    sigrc, (ulong)sig->keyid[1], datestr_from_sig(sig));
+	    printf("%c%c %c%c%c%c%c %08lX %s   ",
+                   sigrc,(sig->sig_class-0x10>0 &&
+                          sig->sig_class-0x10<4)?'0'+sig->sig_class-0x10:' ',
+                   sig->flags.exportable?' ':'L',
+                   sig->flags.revocable?' ':'R',
+                   sig->flags.policy_url?'P':' ',
+                   sig->flags.notation?'N':' ',
+                   sig->flags.expired?'X':' ',
+                   (ulong)sig->keyid[1], datestr_from_sig(sig));
 	    if( sigrc == '%' )
 		printf("[%s] ", g10_errstr(rc) );
 	    else if( sigrc == '?' )
@@ -363,6 +522,13 @@ list_keyblock_print ( KBNODE keyblock, int secret )
 		m_free(p);
 	    }
 	    putchar('\n');
+
+	    if(sig->flags.policy_url && opt.show_policy_url)
+	      show_policy_url(sig,3);
+
+	    if(sig->flags.notation && opt.show_notation)
+	      show_notation(sig,3);
+
 	    /* fixme: check or list other sigs here */
 	}
     }
@@ -371,7 +537,7 @@ list_keyblock_print ( KBNODE keyblock, int secret )
 
 
 static void
-list_keyblock_colon( KBNODE keyblock, int secret )
+list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 {
     int rc = 0;
     KBNODE kbctx;
@@ -418,7 +584,7 @@ list_keyblock_colon( KBNODE keyblock, int secret )
         else if ( opt.fast_list_mode || opt.no_expensive_trust_checks ) 
             ;
         else {
-            trustletter = query_trust_info( pk, NULL );
+            trustletter = get_validity_info ( pk, NULL );
             if( trustletter == 'u' )
                 ulti_hack = 1;
             putchar(trustletter);
@@ -432,9 +598,8 @@ list_keyblock_colon( KBNODE keyblock, int secret )
         if( pk->local_id )
             printf("%lu", pk->local_id );
         putchar(':');
-        if( pk->local_id && !opt.fast_list_mode
-            && !opt.no_expensive_trust_checks  )
-            putchar( get_ownertrust_info( pk->local_id ) );
+        if( !opt.fast_list_mode && !opt.no_expensive_trust_checks  )
+            putchar( get_ownertrust_info(pk) );
 	    putchar(':');
     }
     
@@ -444,8 +609,8 @@ list_keyblock_colon( KBNODE keyblock, int secret )
         putchar(':');
         print_capabilities (pk, sk, keyblock);
         putchar('\n');
-        if( opt.fingerprint )
-            fingerprint( pk, sk );
+        if( fpr )
+            print_fingerprint( pk, sk, 0 );
         if( opt.with_key_data )
             print_key_data( pk, keyid );
         any = 1;
@@ -467,15 +632,15 @@ list_keyblock_colon( KBNODE keyblock, int secret )
 		    byte namehash[20];
 
 		    if( pk && !ulti_hack ) {
-			if( node->pkt->pkt.user_id->photo )
+			if( node->pkt->pkt.user_id->attrib_data )
 			    rmd160_hash_buffer( namehash,
-					    node->pkt->pkt.user_id->photo,
-					    node->pkt->pkt.user_id->photolen);
+					   node->pkt->pkt.user_id->attrib_data,
+					   node->pkt->pkt.user_id->attrib_len);
 			else
 			    rmd160_hash_buffer( namehash,
 					    node->pkt->pkt.user_id->name,
 					    node->pkt->pkt.user_id->len  );
-			trustletter = query_trust_info( pk, namehash );
+			trustletter = get_validity_info( pk, namehash );
 		    }
 		    else
 			trustletter = 'u';
@@ -491,8 +656,8 @@ list_keyblock_colon( KBNODE keyblock, int secret )
                 putchar(':');
                 print_capabilities (pk, sk, keyblock);
                 putchar('\n');
-		if( opt.fingerprint )
-		    fingerprint( pk, sk );
+		if( fpr )
+		    print_fingerprint( pk, sk, 0 );
 		if( opt.with_key_data )
 		    print_key_data( pk, keyid );
 		any = 1;
@@ -507,8 +672,8 @@ list_keyblock_colon( KBNODE keyblock, int secret )
                 putchar(':');
                 print_capabilities (pk, sk, keyblock);
                 putchar('\n');
-		if( opt.fingerprint )
-		    fingerprint( pk, sk ); /* of the main key */
+		if( fpr )
+		    print_fingerprint( pk, sk, 0 ); /* of the main key */
 		any = 1;
 	    }
 
@@ -541,8 +706,8 @@ list_keyblock_colon( KBNODE keyblock, int secret )
             putchar(':');
             print_capabilities (pk2, NULL, NULL);
             putchar('\n');
-	    if( opt.fingerprint > 1 )
-		fingerprint( pk2, NULL );
+	    if( fpr > 1 )
+		print_fingerprint( pk2, NULL, 0 );
 	    if( opt.with_key_data )
 		print_key_data( pk2, keyid2 );
 	}
@@ -555,8 +720,8 @@ list_keyblock_colon( KBNODE keyblock, int secret )
                 putchar(':');
                 print_capabilities (pk, sk, keyblock);
 		putchar('\n');
-		if( opt.fingerprint )
-		    fingerprint( pk, sk ); /* of the main key */
+		if( fpr )
+		    print_fingerprint( pk, sk, 0 ); /* of the main key */
 		any = 1;
 	    }
 
@@ -570,8 +735,8 @@ list_keyblock_colon( KBNODE keyblock, int secret )
                    /* fixme: add LID */ );
             print_capabilities (NULL, sk2, NULL);
             putchar ('\n');
-	    if( opt.fingerprint > 1 )
-		fingerprint( NULL, sk2 );
+	    if( fpr > 1 )
+		print_fingerprint( NULL, sk2, 0 );
 	}
 	else if( opt.list_sigs && node->pkt->pkttype == PKT_SIGNATURE ) {
 	    PKT_signature *sig = node->pkt->pkt.signature;
@@ -590,8 +755,8 @@ list_keyblock_colon( KBNODE keyblock, int secret )
                 putchar(':');
                 print_capabilities (pk, sk, keyblock);
                 putchar('\n');
-		if( opt.fingerprint )
-		    fingerprint( pk, sk );
+		if( fpr )
+		    print_fingerprint( pk, sk, 0 );
 		any=1;
 	    }
 
@@ -604,7 +769,7 @@ list_keyblock_colon( KBNODE keyblock, int secret )
 	       sigstr = "sig";
 	    else {
                 printf ("sig::::::::::%02x%c:\n",
-                        sig->sig_class, get_lsign_letter (sig) );
+                        sig->sig_class, sig->flags.exportable?'x':'l');
 		continue;
 	    }
 	    if( opt.check_sigs ) {
@@ -626,9 +791,10 @@ list_keyblock_colon( KBNODE keyblock, int secret )
             putchar(':');
             if( sigrc != ' ' )
                 putchar(sigrc);
-            printf("::%d:%08lX%08lX:%s::::", sig->pubkey_algo,
+            printf("::%d:%08lX%08lX:%s:%s:::", sig->pubkey_algo,
 						 (ulong)sig->keyid[0],
-			   (ulong)sig->keyid[1], colon_datestr_from_sig(sig));
+			   (ulong)sig->keyid[1], colon_datestr_from_sig(sig),
+		           colon_expirestr_from_sig(sig));
 	    if( sigrc == '%' )
 		printf("[%s] ", g10_errstr(rc) );
 	    else if( sigrc == '?' )
@@ -639,7 +805,7 @@ list_keyblock_colon( KBNODE keyblock, int secret )
                 print_string( stdout, p, n, ':' );
 		m_free(p);
 	    }
-            printf(":%02x%c:\n", sig->sig_class, get_lsign_letter (sig) );
+            printf(":%02x%c:\n", sig->sig_class,sig->flags.exportable?'x':'l');
 	    /* fixme: check or list other sigs here */
 	}
     }
@@ -652,9 +818,9 @@ list_keyblock_colon( KBNODE keyblock, int secret )
 }
 
 /*
- * Reorder the keyblock so that the primary user ID comes first.
- * Fixme: Replace this by a generic sort function.
- */
+ * Reorder the keyblock so that the primary user ID (and not attribute
+ * packet) comes first.  Fixme: Replace this by a generic sort
+ * function.  */
 static void
 reorder_keyblock (KBNODE keyblock)
 {
@@ -662,12 +828,12 @@ reorder_keyblock (KBNODE keyblock)
     KBNODE last, node;
 
     for (node=keyblock; node; primary0=node, node = node->next) {
-	if( node->pkt->pkttype == PKT_USER_ID 
-            && node->pkt->pkt.user_id->is_primary ) {
+	if( node->pkt->pkttype == PKT_USER_ID &&
+	    !node->pkt->pkt.user_id->attrib_data &&
+            node->pkt->pkt.user_id->is_primary ) {
             primary = primary2 = node;
             for (node=node->next; node; primary2=node, node = node->next ) {
                 if( node->pkt->pkttype == PKT_USER_ID 
-                    || node->pkt->pkttype == PKT_PHOTO_ID 
                     || node->pkt->pkttype == PKT_PUBLIC_SUBKEY 
                     || node->pkt->pkttype == PKT_SECRET_SUBKEY ) {
                     break;
@@ -694,48 +860,96 @@ reorder_keyblock (KBNODE keyblock)
     primary2->next = node;
 }
 
-static void
-list_keyblock( KBNODE keyblock, int secret )
+void
+list_keyblock( KBNODE keyblock, int secret, int fpr, void *opaque )
 {
     reorder_keyblock (keyblock);
     if (opt.with_colons)
-        list_keyblock_colon (keyblock, secret );
+        list_keyblock_colon (keyblock, secret, fpr );
     else
-        list_keyblock_print (keyblock, secret );
+        list_keyblock_print (keyblock, secret, fpr, opaque );
 }
 
-static void
-fingerprint( PKT_public_key *pk, PKT_secret_key *sk )
+/*
+ * standard function to print the finperprint.
+ * mode 0: as used in key listings, opt.with_colons is honored
+ *      1: print using log_info ()
+ *      2: direct use of tty
+ */
+void
+print_fingerprint (PKT_public_key *pk, PKT_secret_key *sk, int mode )
 {
-    byte *array, *p;
+    byte array[MAX_FINGERPRINT_LEN], *p;
     size_t i, n;
+    FILE *fp;
+    const char *text;
 
-    p = array = pk? fingerprint_from_pk( pk, NULL, &n )
-		   : fingerprint_from_sk( sk, NULL, &n );
-    if( opt.with_colons ) {
-	printf("fpr:::::::::");
-	for(i=0; i < n ; i++, p++ )
-	    printf("%02X", *p );
-	putchar(':');
+    if (mode == 1) {
+        fp = log_stream ();
+        text = _("Fingerprint:");
+    }
+    else if (mode == 2) {
+        fp = NULL; /* use tty */
+        /* Translators: this should fit into 24 bytes to that the fingerprint
+         * data is properly aligned with the user ID */
+        text = _("             Fingerprint:");
     }
     else {
-	printf("     Key fingerprint =");
-	if( n == 20 ) {
-	    for(i=0; i < n ; i++, i++, p += 2 ) {
-		if( i == 10 )
-		    putchar(' ');
-		printf(" %02X%02X", *p, p[1] );
+        fp = stdout;
+        text = _("     Key fingerprint =");
+    }
+  
+    if (sk)
+	fingerprint_from_sk (sk, array, &n);
+    else
+	fingerprint_from_pk (pk, array, &n);
+    p = array;
+    if (opt.with_colons && !mode) {
+	fprintf (fp, "fpr:::::::::");
+	for (i=0; i < n ; i++, p++ )
+	    fprintf (fp, "%02X", *p );
+	putc(':', fp);
+    }
+    else {
+        if (fp)
+            fputs (text, fp);
+        else
+            tty_printf ("%s", text);
+	if (n == 20) {
+	    for (i=0; i < n ; i++, i++, p += 2 ) {
+                if (fp) {
+                    if (i == 10 )
+                        putc(' ', fp);
+                    fprintf (fp, " %02X%02X", *p, p[1] );
+                }
+                else {
+                    if (i == 10 )
+                        tty_printf (" ");
+                    tty_printf (" %02X%02X", *p, p[1]);
+                }
 	    }
 	}
 	else {
-	    for(i=0; i < n ; i++, p++ ) {
-		if( i && !(i%8) )
-		    putchar(' ');
-		printf(" %02X", *p );
+	    for (i=0; i < n ; i++, p++ ) {
+                if (fp) {
+                    if (i && !(i%8) )
+                        putc (' ', fp);
+                    fprintf (fp, " %02X", *p );
+                }
+                else {
+                    if (i && !(i%8) )
+                        tty_printf (" ");
+                    tty_printf (" %02X", *p );
+                }
 	    }
 	}
     }
-    putchar('\n');
-    m_free(array);
+    if (fp)
+        putc ('\n', fp);
+    else
+        tty_printf ("\n");
 }
 
+
+
+
