@@ -1,5 +1,5 @@
 /* pkclist.c
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -96,18 +96,20 @@ do_show_revocation_reason( PKT_signature *sig )
     }
 }
 
+/* Mode 0: try and find the revocation based on the pk (i.e. check
+   subkeys, etc.)  Mode 1: use only the revocation on the main pk */
 
 static void
-show_revocation_reason( PKT_public_key *pk )
+show_revocation_reason( PKT_public_key *pk, int mode )
 {
     /* Hmmm, this is not so easy becuase we have to duplicate the code
      * used in the trustbd to calculate the keyflags.  We need to find
-     * a clean way to check revocation certificates on keys and signatures.
-     * And there should be no duplicate code.  Because we enter this function
-     * only when the trustdb toldus, taht we have a revoked key, we could
-     * simplylook for a revocation cert and display this one, when there is
-     * only one. Let's try to do this until we have a better solution.
-     */
+     * a clean way to check revocation certificates on keys and
+     * signatures.  And there should be no duplicate code.  Because we
+     * enter this function only when the trustdb told us that we have
+     * a revoked key, we could simply look for a revocation cert and
+     * display this one, when there is only one. Let's try to do this
+     * until we have a better solution.  */
     KBNODE node, keyblock = NULL;
     byte fingerprint[MAX_FINGERPRINT_LEN];
     size_t fingerlen;
@@ -122,9 +124,10 @@ show_revocation_reason( PKT_public_key *pk )
     }
 
     for( node=keyblock; node; node = node->next ) {
-	if( ( node->pkt->pkttype == PKT_PUBLIC_KEY
+        if( (mode && node->pkt->pkttype == PKT_PUBLIC_KEY) ||
+	  ( ( node->pkt->pkttype == PKT_PUBLIC_KEY
 	      || node->pkt->pkttype == PKT_PUBLIC_SUBKEY )
-	    && !cmp_public_keys( node->pkt->pkt.public_key, pk ) )
+	    && !cmp_public_keys( node->pkt->pkt.public_key, pk ) ) )
 	    break;
     }
     if( !node ) {
@@ -141,9 +144,14 @@ show_revocation_reason( PKT_public_key *pk )
 		|| node->pkt->pkt.signature->sig_class == 0x28 ) ) {
 		/* FIXME: we should check the signature here */
 		do_show_revocation_reason ( node->pkt->pkt.signature );
+		break;
 	}
     }
 
+    /* We didn't find it, so check if the whole key is revoked */
+    if(!node && !mode)
+      show_revocation_reason(pk,1);
+
     release_kbnode( keyblock );
 }
 
@@ -241,7 +249,7 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
   keyid_from_pk (pk, keyid);
   for(;;) {
     /* a string with valid answers */
-    const char *ans = _("sSmMqQ");
+    const char *ans = _("iImMqQsS");
 
     if( !did_help ) 
       {
@@ -268,15 +276,18 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
         tty_printf (_(" %d = I trust fully\n"), 4);
         if (mode)
           tty_printf (_(" %d = I trust ultimately\n"), 5);
-        tty_printf (_(" s = please show me more information\n") );
+        tty_printf (_(" i = please show me more information\n") );
         if( mode )
           tty_printf(_(" m = back to the main menu\n"));
         else
-          tty_printf(_(" q = quit\n"));
+	  {
+	    tty_printf(_(" s = skip this key\n"));
+	    tty_printf(_(" q = quit\n"));
+	  }
         tty_printf("\n");
         did_help = 1;
       }
-    if( strlen(ans) != 6 )
+    if( strlen(ans) != 8 )
       BUG();
     p = cpr_get("edit_ownertrust.value",_("Your decision? "));
     trim_spaces(p);
@@ -319,6 +330,10 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
       {
         break ; /* back to the menu */
       }
+    else if( !mode && (*p == ans[6] || *p == ans[7] ) )
+      {
+	break; /* skip */
+      }
     else if( !mode && (*p == ans[4] || *p == ans[5] ) )
       {
         quit = 1;
@@ -346,7 +361,7 @@ edit_ownertrust (PKT_public_key *pk, int mode )
       switch ( do_edit_ownertrust (pk, mode, &trust, no_help ) )
         {
         case -1: /* quit */
-          return 0;
+          return -1;
         case -2: /* show info */
           show_paths(pk, 1);
           no_help = 1;
@@ -355,7 +370,7 @@ edit_ownertrust (PKT_public_key *pk, int mode )
           trust &= ~TRUST_FLAG_DISABLED;
           trust |= get_ownertrust (pk) & TRUST_FLAG_DISABLED;
           update_ownertrust (pk, trust );
-          return 0;
+          return 1;
         default:
           return 0;
         }
@@ -383,7 +398,7 @@ do_we_trust( PKT_public_key *pk, unsigned int *trustlevel )
     if( (*trustlevel & TRUST_FLAG_REVOKED) ) {
 	log_info(_("key %08lX: key has been revoked!\n"),
 					(ulong)keyid_from_pk( pk, NULL) );
-	show_revocation_reason( pk );
+	show_revocation_reason( pk, 0 );
 	if( opt.batch )
           return 0; /* no */
 
@@ -395,7 +410,7 @@ do_we_trust( PKT_public_key *pk, unsigned int *trustlevel )
     if( (*trustlevel & TRUST_FLAG_SUB_REVOKED) ) {
 	log_info(_("key %08lX: subkey has been revoked!\n"),
 					(ulong)keyid_from_pk( pk, NULL) );
-	show_revocation_reason( pk );
+	show_revocation_reason( pk, 0 );
 	if( opt.batch )
 	    return 0;
 
@@ -520,7 +535,6 @@ check_signatures_trust( PKT_signature *sig )
 {
   PKT_public_key *pk = m_alloc_clear( sizeof *pk );
   unsigned int trustlevel;
-  int did_add = 0;
   int rc=0;
 
   if ( opt.always_trust)
@@ -547,13 +561,13 @@ check_signatures_trust( PKT_signature *sig )
       write_status( STATUS_KEYREVOKED );
       log_info(_("WARNING: This key has been revoked by its owner!\n"));
       log_info(_("         This could mean that the signature is forgery.\n"));
-      show_revocation_reason( pk );
+      show_revocation_reason( pk, 0 );
     }
   else if ((trustlevel & TRUST_FLAG_SUB_REVOKED) ) 
     {
       write_status( STATUS_KEYREVOKED );
       log_info(_("WARNING: This subkey has been revoked by its owner!\n"));
-      show_revocation_reason( pk );
+      show_revocation_reason( pk, 0 );
     }
   
   if ((trustlevel & TRUST_FLAG_DISABLED))
@@ -572,14 +586,12 @@ check_signatures_trust( PKT_signature *sig )
       /* fall thru */
     case TRUST_UNKNOWN: 
     case TRUST_UNDEFINED:
-      if( did_add || opt.batch || opt.answer_no ) {
-        write_status( STATUS_TRUST_UNDEFINED );
-        log_info(_("WARNING: This key is not certified with"
-                   " a trusted signature!\n"));
-        log_info(_("         There is no indication that the "
-                   "signature belongs to the owner.\n" ));
-        print_fingerprint (pk, NULL, 1);
-      }
+      write_status( STATUS_TRUST_UNDEFINED );
+      log_info(_("WARNING: This key is not certified with"
+                 " a trusted signature!\n"));
+      log_info(_("         There is no indication that the "
+                 "signature belongs to the owner.\n" ));
+      print_fingerprint (pk, NULL, 1);
       break;
 
     case TRUST_NEVER:
@@ -730,14 +742,14 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
     }
 
     if( !any_recipients && !opt.batch ) { /* ask */
-	char *answer=NULL;
 	int have_def_rec;
+	char *answer=NULL;
 
 	def_rec = default_recipient();
 	have_def_rec = !!def_rec;
 	if( !have_def_rec )
 	    tty_printf(_(
-		"You did not specify a user ID. (you may use \"-r\")\n\n"));
+		"You did not specify a user ID. (you may use \"-r\")\n"));
 	for(;;) {
 	    rc = 0;
 	    m_free(answer);
@@ -747,12 +759,14 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 	    }
 	    else {
 		answer = cpr_get_utf8("pklist.user_id.enter",
-				       _("Enter the user ID: "));
+			 _("\nEnter the user ID.  End with an empty line: "));
 		trim_spaces(answer);
 		cpr_kill_prompt();
 	    }
-	    if( !*answer )
+	    if( !answer || !*answer ) {
+	        m_free(answer);
 		break;
+	    }
 	    if( pk )
 		free_public_key( pk );
 	    pk = m_alloc_clear( sizeof *pk );
@@ -775,7 +789,7 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 			pk_list = r;
 		    }
 		    any_recipients = 1;
-		    break;
+		    continue;
 		}
 		else {
 		    int trustlevel;
@@ -789,11 +803,24 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 			 * in the list */
 			if (key_present_in_pk_list(pk_list, pk) == 0) {
 			    free_public_key(pk); pk = NULL;
-			    log_info(_("skipped: public key "
-				       "already set with --encrypt-to\n") );
+			    log_info(_("skipped: public key already set\n") );
 			}
 			else {
 			    PK_LIST r;
+			    char *p;
+			    size_t n;
+			    u32 keyid[2];
+
+			    keyid_from_pk( pk, keyid);
+			    tty_printf("Added %4u%c/%08lX %s \"",
+				       nbits_from_pk( pk ),
+				       pubkey_letter( pk->pubkey_algo ),
+				       (ulong)keyid[1],
+				       datestr_from_pk( pk ) );
+			    p = get_user_id( keyid, &n );
+			    tty_print_utf8_string( p, n );
+			    m_free(p);
+			    tty_printf("\"\n");
 
 			    r = m_alloc( sizeof *r );
 			    r->pk = pk; pk = NULL;
@@ -802,14 +829,13 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 			    pk_list = r;
 			}
 			any_recipients = 1;
-			break;
+			continue;
 		    }
 		}
 	    }
 	    m_free(def_rec); def_rec = NULL;
 	    have_def_rec = 0;
 	}
-	m_free(answer);
 	if( pk ) {
 	    free_public_key( pk );
 	    pk = NULL;
@@ -822,12 +848,20 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 	if( rc )
 	    log_error(_("unknown default recipient `%s'\n"), def_rec );
 	else if( !(rc=check_pubkey_algo2(pk->pubkey_algo, use)) ) {
+	  /* Mark any_recipients here since the default recipient
+             would have been used if it wasn't already there.  It
+             doesn't really matter if we got this key from the default
+             recipient or an encrypt-to. */
+	  any_recipients = 1;
+	  if (key_present_in_pk_list(pk_list, pk) == 0)
+	    log_info(_("skipped: public key already set as default recipient\n"));
+	  else {
 	    PK_LIST r = m_alloc( sizeof *r );
 	    r->pk = pk; pk = NULL;
 	    r->next = pk_list;
 	    r->mark = 0;
 	    pk_list = r;
-	    any_recipients = 1;
+	  }
 	}
 	if( pk ) {
 	    free_public_key( pk );
@@ -919,17 +953,40 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 }
 
 
+/* In pgp6 mode, disallow all ciphers except IDEA (1), 3DES (2), and
+   CAST5 (3), all hashes except MD5 (1), SHA1 (2), and RIPEMD160 (3),
+   and all compressions except none (0) and ZIP (1).  For a true PGP6
+   key all of this is unneeded as they are the only items present in
+   the preferences subpacket, but checking here covers the weird case
+   of encrypting to a key that had preferences from a different
+   implementation which was then used with PGP6.  I am not completely
+   comfortable with this as the right thing to do, as it slightly
+   alters the list of what the user is supposedly requesting.  It is
+   not against the RFC however, as the preference chosen will never be
+   one that the user didn't specify somewhere ("The implementation may
+   use any mechanism to pick an algorithm in the intersection"), and
+   PGP6 has no mechanism to fix such a broken preference list, so I'm
+   including it. -dms */
 
 static int
 algo_available( int preftype, int algo )
 {
     if( preftype == PREFTYPE_SYM ) {
+        if( opt.pgp6 && ( algo != 1 && algo != 2 && algo != 3) )
+	  return 0;
+
 	return algo && !check_cipher_algo( algo );
     }
     else if( preftype == PREFTYPE_HASH ) {
+        if( opt.pgp6 && ( algo != 1 && algo != 2 && algo != 3) )
+	  return 0;
+
 	return algo && !check_digest_algo( algo );
     }
     else if( preftype == PREFTYPE_ZIP ) {
+        if ( opt.pgp6 && ( algo !=0 && algo != 1) )
+	  return 0;
+
 	return !algo || algo == 1 || algo == 2;
     }
     else
@@ -959,8 +1016,18 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
 	u32 mask[8];
 
 	memset( mask, 0, 8 * sizeof *mask );
-	if( preftype == PREFTYPE_SYM )
-	    mask[0] |= (1<<2); /* 3DES is implicitly there */
+	if( preftype == PREFTYPE_SYM ) {
+	  if( pkr->pk->version < 4 &&
+	      pkr->pk->selfsigversion < 4 &&
+	      opt.pgp2 )
+	    mask[0] |= (1<<1); /* IDEA is implicitly there for v3 keys
+				  with v3 selfsigs (rfc2440:12.1) if
+				  --pgp2 mode is on.  This doesn't
+				  mean it's actually available, of
+				  course. */
+	  else
+	    mask[0] |= (1<<2); /* 3DES is implicitly there for everyone else */
+	}
         
         if (pkr->pk->user_id) /* selected by user ID */
             prefs = pkr->pk->user_id->prefs;
@@ -1063,5 +1130,3 @@ select_mdc_from_pklist (PK_LIST pk_list)
     }
     return 1; /* can be used */
 }
-
-
