@@ -1,5 +1,5 @@
 /* revoke.c
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -52,6 +52,9 @@ revocation_reason_build_cb( PKT_signature *sig, void *opaque )
     byte *buffer;
     size_t buflen = 1;
 
+    if(!reason)
+      return 0;
+
     if( reason->desc ) {
 	ud = native_to_utf8( reason->desc );
 	buflen += strlen(ud);
@@ -159,11 +162,13 @@ gen_revoke( const char *uname )
 	goto leave;
     }
 
-    /* get the reason for the revocation */
-    reason = ask_revocation_reason( 1, 0, 1 );
-    if( !reason ) { /* user decided to cancel */
+    if(sk->version>=4 || opt.force_v4_certs) {
+      /* get the reason for the revocation */
+      reason = ask_revocation_reason( 1, 0, 1 );
+      if( !reason ) { /* user decided to cancel */
 	rc = 0;
 	goto leave;
+      }
     }
 
     switch( is_secret_key_protected( sk ) ) {
@@ -193,9 +198,9 @@ gen_revoke( const char *uname )
     iobuf_push_filter( out, armor_filter, &afx );
 
     /* create it */
-    rc = make_keysig_packet( &sig, pk, NULL, NULL, sk, 0x20, 0, 0,
-						  revocation_reason_build_cb,
-						  reason );
+    rc = make_keysig_packet( &sig, pk, NULL, NULL, sk, 0x20, 0,
+			     opt.force_v4_certs?4:0, 0, 0,
+			     revocation_reason_build_cb, reason );
     if( rc ) {
 	log_error(_("make_keysig_packet failed: %s\n"), g10_errstr(rc));
 	goto leave;
@@ -240,9 +245,10 @@ gen_revoke( const char *uname )
 struct revocation_reason_info *
 ask_revocation_reason( int key_rev, int cert_rev, int hint )
 {
-    int code;
+    int code=-1;
     char *description = NULL;
     struct revocation_reason_info *reason;
+    const char *text_0 = _("No reason specified");
     const char *text_1 = _("Key has been compromised");
     const char *text_2 = _("Key is superseded");
     const char *text_3 = _("Key is no longer used");
@@ -254,6 +260,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	description = NULL;
 
 	tty_printf(_("Please select the reason for the revocation:\n"));
+	tty_printf(    "  0 = %s\n", text_0 );
 	if( key_rev )
 	    tty_printf("  1 = %s\n", text_1 );
 	if( key_rev )
@@ -262,29 +269,31 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	    tty_printf("  3 = %s\n", text_3 );
 	if( cert_rev )
 	    tty_printf("  4 = %s\n", text_4 );
-	tty_printf(    "  0 = %s\n", _("Cancel") );
+	tty_printf(    "  Q = %s\n", _("Cancel") );
 	if( hint )
 	    tty_printf(_("(Probably you want to select %d here)\n"), hint );
 
-	for(code = 0; !code;) {
+	while(code==-1) {
 	    int n;
 	    char *answer = cpr_get("ask_revocation_reason.code",
 						_("Your decision? "));
 	    trim_spaces( answer );
 	    cpr_kill_prompt();
-	    if( *answer == 'q' || *answer == 'Q' )
-		n = 0;
-	    else if( !isdigit( *answer ) )
-		n = -1;
-	    else if( hint && !*answer )
+	    if( *answer == 'q' || *answer == 'Q')
+	      return NULL; /* cancel */
+	    if( hint && !*answer )
 		n = hint;
+	    else if(!isdigit( *answer ) )
+ 	        n = -1;
 	    else
 		n = atoi(answer);
 	    m_free(answer);
-	    if( !n )
-		return NULL; /* cancel */
+	    if( n == 0 ) {
+	        code = 0x00; /* no particular reason */
+		code_text = text_0;
+	    }
 	    else if( key_rev && n == 1 ) {
-		code = 0x02; /* key has  been compromised */
+		code = 0x02; /* key has been compromised */
 		code_text = text_1;
 	    }
 	    else if( key_rev && n == 2 ) {
