@@ -1,5 +1,5 @@
 /* seckey-cert.c -  secret key certificate packet handling
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -36,7 +36,7 @@
 
 
 static int
-do_check( PKT_secret_key *sk )
+do_check( PKT_secret_key *sk, const char *tryagain_text )
 {
     byte *buffer;
     u16 csum=0;
@@ -56,8 +56,9 @@ do_check( PKT_secret_key *sk )
 	if( sk->protect.algo == CIPHER_ALGO_NONE )
 	    BUG();
 	if( check_cipher_algo( sk->protect.algo ) ) {
-	    log_info(_("protection algorithm %d is not supported\n"),
-			sk->protect.algo );
+	    log_info(_("protection algorithm %d%s is not supported\n"),
+			sk->protect.algo,sk->protect.algo==1?" (IDEA)":"" );
+	    idea_cipher_warn(0);
 	    return G10ERR_CIPHER_ALGO;
 	}
 	keyid_from_sk( sk, keyid );
@@ -67,7 +68,7 @@ do_check( PKT_secret_key *sk )
             keyid[3] = sk->main_keyid[1];
 	}
 	dek = passphrase_to_dek( keyid, sk->pubkey_algo, sk->protect.algo,
-				 &sk->protect.s2k, 0 );
+				 &sk->protect.s2k, 0, tryagain_text );
 	cipher_hd = cipher_open( sk->protect.algo,
 				 CIPHER_MODE_AUTO_CFB, 1);
 	cipher_setkey( cipher_hd, dek->key, dek->keylen );
@@ -89,31 +90,60 @@ do_check( PKT_secret_key *sk )
 	    cipher_decrypt( cipher_hd, data, p, ndata );
 	    mpi_free( sk->skey[i] ); sk->skey[i] = NULL ;
 	    p = data;
-	    if( ndata < 2 ) {
-		log_error("not enough bytes for checksum\n");
-		sk->csum = 0;
-		csum = 1;
-	    }
-	    else {
-		csum = checksum( data, ndata-2);
-		sk->csum = data[ndata-2] << 8 | data[ndata-1];
-                if ( sk->csum != csum ) {
-                    /* This is a PGP 7.0.0 workaround */
-                    sk->csum = csumc; /* take the encrypted one */
+            if (sk->protect.sha1chk) {
+                /* This is the new SHA1 checksum method to detect
+                   tampering with the key as used by the Klima/Rosa
+                   attack */
+                sk->csum = 0;
+                csum = 1;
+                if( ndata < 20 ) 
+                    log_error("not enough bytes for SHA-1 checksum\n");
+                else {
+                    MD_HANDLE h = md_open (DIGEST_ALGO_SHA1, 1);
+                    if (!h)
+                        BUG(); /* algo not available */
+                    md_write (h, data, ndata - 20);
+                    md_final (h);
+                    if (!memcmp (md_read (h, DIGEST_ALGO_SHA1),
+                                 data + ndata - 20, 20) ) {
+                        /* digest does match.  We have to keep the old
+                           style checksum in sk->csum, so that the
+                           test used for unprotected keys does work.
+                           This test gets used when we are adding new
+                           keys. */
+                        sk->csum = csum = checksum (data, ndata-20);
+                    }
+                    md_close (h);
                 }
-	    }
-            
-	    /* must check it here otherwise the mpi_read_xx would fail
-	     * because the length may have an arbitrary value */
-	    if( sk->csum == csum ) {
-		for( ; i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
-		    nbytes = ndata;
-		    sk->skey[i] = mpi_read_from_buffer(p, &nbytes, 1 );
-		    ndata -= nbytes;
-		    p += nbytes;
-		}
-                /* at this point ndata should be equal to 2 (the checksum) */
-	    }
+            }
+            else {
+                if( ndata < 2 ) {
+                    log_error("not enough bytes for checksum\n");
+                    sk->csum = 0;
+                    csum = 1;
+                }
+                else {
+                    csum = checksum( data, ndata-2);
+                    sk->csum = data[ndata-2] << 8 | data[ndata-1];
+                    if ( sk->csum != csum ) {
+                        /* This is a PGP 7.0.0 workaround */
+                        sk->csum = csumc; /* take the encrypted one */
+                    }
+                }
+            }
+                
+            /* must check it here otherwise the mpi_read_xx would fail
+               because the length may have an arbitrary value */
+            if( sk->csum == csum ) {
+                for( ; i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
+                    nbytes = ndata;
+                    sk->skey[i] = mpi_read_from_buffer(p, &nbytes, 1 );
+                    ndata -= nbytes;
+                    p += nbytes;
+                }
+                /* Note: at this point ndata should be 2 for a simple
+                   checksum or 20 for the sha1 digest */
+            }
 	    m_free(data);
 	}
 	else {
@@ -177,12 +207,15 @@ check_secret_key( PKT_secret_key *sk, int n )
     int i;
 
     if( n < 1 )
-	n = opt.batch? 1 : 3; /* use the default value */
+	n = (opt.batch && !opt.use_agent)? 1 : 3; /* use the default value */
 
     for(i=0; i < n && rc == G10ERR_BAD_PASS; i++ ) {
-	if( i )
-	    log_info(_("Invalid passphrase; please try again ...\n"));
-	rc = do_check( sk );
+        const char *tryagain = NULL;
+	if (i) {
+            tryagain = _("Invalid passphrase; please try again");
+            log_info (_("%s ...\n"), tryagain);
+        }
+	rc = do_check( sk, tryagain );
 	if( rc == G10ERR_BAD_PASS && is_status_enabled() ) {
 	    u32 kid[2];
 	    char buf[50];
@@ -261,7 +294,7 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 		}
 		for( ; j < PUBKEY_MAX_NSKEY; j++ )
 		    bufarr[j] = NULL;
-		ndata += 2; /* for checksum */
+		ndata += opt.simple_sk_checksum? 2 : 20; /* for checksum */
 
 		data = m_alloc_secure( ndata );
 		p = data;
@@ -273,11 +306,30 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 		    p += narr[j];
 		    m_free(bufarr[j]);
 		}
-		csum = checksum( data, ndata-2);
-		sk->csum = csum;
-		*p++ =	csum >> 8;
-		*p++ =	csum;
-		assert( p == data+ndata );
+                
+                if (opt.simple_sk_checksum) {
+                    log_info (_("generating the deprecated 16-bit checksum"
+                              " for secret key protection\n")); 
+                    csum = checksum( data, ndata-2);
+                    sk->csum = csum;
+                    *p++ =	csum >> 8;
+                    *p++ =	csum;
+                    sk->protect.sha1chk = 0;
+                }
+                else {
+                    MD_HANDLE h = md_open (DIGEST_ALGO_SHA1, 1);
+                    if (!h)
+                        BUG(); /* algo not available */
+                    md_write (h, data, ndata - 20);
+                    md_final (h);
+                    memcpy (p, md_read (h, DIGEST_ALGO_SHA1), 20);
+                    p += 20;
+                    md_close (h);
+                    sk->csum = csum = 0;
+                    sk->protect.sha1chk = 1;
+                }
+                assert( p == data+ndata );
+
 		cipher_encrypt( cipher_hd, data, data, ndata );
 		for(i = pubkey_get_npkey(sk->pubkey_algo);
 			i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
