@@ -1,5 +1,5 @@
 /* sign.c - sign data
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,6 +24,7 @@
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
+#include <unistd.h> /* need sleep() */
 
 #include "options.h"
 #include "packet.h"
@@ -42,20 +43,22 @@
 
 #ifdef HAVE_DOSISH_SYSTEM
   #define LF "\r\n"
+  void __stdcall Sleep(ulong);
+  #define sleep(a)  Sleep((a)*1000)
 #else
   #define LF "\n"
 #endif
 
-
 /****************
  * Create a notation.  It is assumed that the stings in STRLIST
  * are already checked to contain only printable data and have a valid
  * NAME=VALUE format.
  */
 static void
-mk_notation_and_policy( PKT_signature *sig )
+mk_notation_and_policy( PKT_signature *sig, PKT_public_key *pk )
 {
-    const char *string, *s;
+    const char *string;
+    char *s=NULL;
     byte *buf;
     unsigned n1, n2;
 
@@ -85,16 +88,129 @@ mk_notation_and_policy( PKT_signature *sig )
 	    build_sig_subpkt( sig, SIGSUBPKT_NOTATION
 			      | ((nd->flags & 1)? SIGSUBPKT_FLAG_CRITICAL:0),
 			      buf, 8+n1+n2 );
+
+	    if(opt.show_notation)
+	      show_notation(sig,0);
 	}
     }
 
     /* set policy URL */
-    if( (s=opt.set_policy_url) ) {
+    if( IS_SIG(sig) && opt.sig_policy_url )
+      {
+	if(sig->version<4)
+	  log_info("can't put a policy URL into v3 signatures\n");
+	else
+	  s=m_strdup(opt.sig_policy_url);
+      }
+    else if( IS_CERT(sig) && opt.cert_policy_url )
+      {
+	if(sig->version<4)
+	  log_info("can't put a policy URL into v3 key signatures\n");
+	else
+	  if(pk)
+	    {
+	      s=pct_expando(opt.cert_policy_url,pk);
+	      if(!s)
+		{
+		  log_error(_("WARNING: unable to %%-expand policy url "
+			      "(too large).  Using unexpanded.\n"));
+		  s=m_strdup(opt.cert_policy_url);
+		}
+	    }
+	  else
+	    s=m_strdup(opt.cert_policy_url);
+      }
+
+    if( s ) {
 	if( *s == '!' )
 	    build_sig_subpkt( sig, SIGSUBPKT_POLICY | SIGSUBPKT_FLAG_CRITICAL,
 			      s+1, strlen(s+1) );
 	else
 	    build_sig_subpkt( sig, SIGSUBPKT_POLICY, s, strlen(s) );
+
+	if(opt.show_policy_url)
+	  show_policy_url(sig,0);
+    }
+
+    m_free(s);
+}
+
+
+/*
+ * Helper to hash a user ID packet.  
+ */
+static void
+hash_uid (MD_HANDLE md, int sigversion, const PKT_user_id *uid)
+{
+    if ( sigversion >= 4 ) {
+        byte buf[5];
+
+	if(uid->attrib_data) {
+	  buf[0] = 0xd1;	           /* indicates an attribute packet */
+	  buf[1] = uid->attrib_len >> 24;  /* always use 4 length bytes */
+	  buf[2] = uid->attrib_len >> 16;
+	  buf[3] = uid->attrib_len >>  8;
+	  buf[4] = uid->attrib_len;
+	}
+	else {
+	  buf[0] = 0xb4;	    /* indicates a userid packet */
+	  buf[1] = uid->len >> 24;  /* always use 4 length bytes */
+	  buf[2] = uid->len >> 16;
+	  buf[3] = uid->len >>  8;
+	  buf[4] = uid->len;
+	}
+        md_write( md, buf, 5 );
+    }
+
+    if(uid->attrib_data)
+      md_write (md, uid->attrib_data, uid->attrib_len );
+    else
+      md_write (md, uid->name, uid->len );
+}
+
+
+/*
+ * Helper to hash some parts from the signature
+ */
+static void
+hash_sigversion_to_magic (MD_HANDLE md, const PKT_signature *sig)
+{
+    if (sig->version >= 4) 
+        md_putc (md, sig->version);
+    md_putc (md, sig->sig_class);
+    if (sig->version < 4) {
+        u32 a = sig->timestamp;
+        md_putc (md, (a >> 24) & 0xff );
+        md_putc (md, (a >> 16) & 0xff );
+        md_putc (md, (a >>  8) & 0xff );
+        md_putc (md,  a	       & 0xff );
+    }
+    else {
+        byte buf[6];
+        size_t n;
+        
+        md_putc (md, sig->pubkey_algo);
+        md_putc (md, sig->digest_algo);
+        if (sig->hashed) {
+            n = sig->hashed->len;
+            md_putc (md, (n >> 8) );
+            md_putc (md,  n       );
+            md_write (md, sig->hashed->data, n );
+            n += 6;
+        }
+        else {
+            md_putc (md, 0);  /* always hash the length of the subpacket*/
+            md_putc (md, 0);
+            n = 6;
+        }
+        /* add some magic */
+        buf[0] = sig->version;
+        buf[1] = 0xff;
+        buf[2] = n >> 24; /* hmmm, n is only 16 bit, so this is always 0 */
+        buf[3] = n >> 16;
+        buf[4] = n >>  8;
+        buf[5] = n;
+        md_write (md, buf, 6);
     }
 }
 
@@ -130,6 +246,8 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
     sig->digest_start[1] = dp[1];
     frame = encode_md_value( sk->pubkey_algo, md,
 			     digest_algo, mpi_get_nbits(sk->skey[0]), 0 );
+    if (!frame)
+        return G10ERR_GENERAL;
     rc = pubkey_sign( sk->pubkey_algo, sig->data, frame, sk->skey );
     mpi_free(frame);
     if (!rc && !opt.no_sig_create_check) {
@@ -144,8 +262,12 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
             frame = encode_md_value (pk->pubkey_algo, md,
                                      sig->digest_algo,
                                      mpi_get_nbits(pk->pkey[0]), 0);
-            rc = pubkey_verify (pk->pubkey_algo, frame, sig->data, pk->pkey,
-                                NULL, NULL );
+            if (!frame)
+                rc = G10ERR_GENERAL;
+            else
+                rc = pubkey_verify (pk->pubkey_algo, frame,
+                                    sig->data, pk->pkey,
+                                    NULL, NULL );
             mpi_free (frame);
         }
         if (rc)
@@ -228,13 +350,217 @@ print_status_sig_created ( PKT_secret_key *sk, PKT_signature *sig, int what )
 }
 
 
+/*
+ * Loop over the secret certificates in SK_LIST and build the one pass
+ * signature packets.  OpenPGP says that the data should be bracket by
+ * the onepass-sig and signature-packet; so we build these onepass
+ * packet here in reverse order 
+ */
+static int
+write_onepass_sig_packets (SK_LIST sk_list, IOBUF out, int sigclass )
+{
+    int skcount;
+    SK_LIST sk_rover;
+
+    for (skcount=0, sk_rover=sk_list; sk_rover; sk_rover = sk_rover->next)
+        skcount++;
+
+    for (; skcount; skcount--) {
+        PKT_secret_key *sk;
+        PKT_onepass_sig *ops;
+        PACKET pkt;
+        int i, rc;
+        
+        for (i=0, sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+            if (++i == skcount)
+                break;
+        }
+
+        sk = sk_rover->sk;
+        ops = m_alloc_clear (sizeof *ops);
+        ops->sig_class = sigclass;
+        ops->digest_algo = hash_for (sk->pubkey_algo, sk->version);
+        ops->pubkey_algo = sk->pubkey_algo;
+        keyid_from_sk (sk, ops->keyid);
+        ops->last = (skcount == 1);
+        
+        init_packet(&pkt);
+        pkt.pkttype = PKT_ONEPASS_SIG;
+        pkt.pkt.onepass_sig = ops;
+        rc = build_packet (out, &pkt);
+        free_packet (&pkt);
+        if (rc) {
+            log_error ("build onepass_sig packet failed: %s\n",
+                       g10_errstr(rc));
+            return rc;
+        }
+    }
+
+    return 0;
+}
+
+/*
+ * Helper to write the plaintext (literal data) packet
+ */
+static int
+write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
+{
+    PKT_plaintext *pt = NULL;
+    u32 filesize;
+    int rc = 0;
+
+    if (!opt.no_literal) {
+        if (fname || opt.set_filename) {
+            char *s = make_basename (opt.set_filename? opt.set_filename
+                                                     : fname);
+            pt = m_alloc (sizeof *pt + strlen(s) - 1);
+            pt->namelen = strlen (s);
+            memcpy (pt->name, s, pt->namelen);
+            m_free (s);
+        }
+        else { /* no filename */
+            pt = m_alloc (sizeof *pt - 1);
+            pt->namelen = 0;
+        }
+    }
+
+    /* try to calculate the length of the data */
+    if (fname) {
+        if( !(filesize = iobuf_get_filelength(inp)) )
+            log_info (_("WARNING: `%s' is an empty file\n"), fname);
+
+        /* we can't yet encode the length of very large files,
+         * so we switch to partial length encoding in this case */
+        if (filesize >= IOBUF_FILELENGTH_LIMIT)
+            filesize = 0;
+
+        /* because the text_filter modifies the length of the
+         * data, it is not possible to know the used length
+         * without a double read of the file - to avoid that
+         * we simple use partial length packets.
+         */
+        if ( ptmode == 't' )
+            filesize = 0;
+    }
+    else {
+        filesize = opt.set_filesize? opt.set_filesize : 0; /* stdin */
+    }
+
+    if (!opt.no_literal) {
+        PACKET pkt;
+
+        pt->timestamp = make_timestamp ();
+        pt->mode = ptmode;
+        pt->len = filesize;
+        pt->new_ctb = !pt->len && !opt.rfc1991;
+        pt->buf = inp;
+        init_packet(&pkt);
+        pkt.pkttype = PKT_PLAINTEXT;
+        pkt.pkt.plaintext = pt;
+        /*cfx.datalen = filesize? calc_packet_length( &pkt ) : 0;*/
+        if( (rc = build_packet (out, &pkt)) )
+            log_error ("build_packet(PLAINTEXT) failed: %s\n",
+                       g10_errstr(rc) );
+        pt->buf = NULL;
+    }
+    else {
+        byte copy_buffer[4096];
+        int  bytes_copied;
+
+        while ((bytes_copied = iobuf_read(inp, copy_buffer, 4096)) != -1)
+            if (iobuf_write(out, copy_buffer, bytes_copied) == -1) {
+                rc = G10ERR_WRITE_FILE;
+                log_error ("copying input to output failed: %s\n",
+                           g10_errstr(rc));
+                break;
+            }
+        memset(copy_buffer, 0, 4096); /* burn buffer */
+    }
+    /* fixme: it seems that we never freed pt/pkt */
+    
+    return rc;
+}
+
+/*
+ * Write the signatures from the SK_LIST to OUT. HASH must be a non-finalized
+ * hash which will not be changes here.
+ */
+static int
+write_signature_packets (SK_LIST sk_list, IOBUF out, MD_HANDLE hash,
+                         int sigclass, u32 timestamp, u32 duration,
+			 int old_style, int status_letter)
+{
+    SK_LIST sk_rover;
+
+    /* loop over the secret certificates */
+    for (sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next) {
+	PKT_secret_key *sk;
+	PKT_signature *sig;
+	MD_HANDLE md;
+        int rc;
+
+	sk = sk_rover->sk;
+
+	/* build the signature packet */
+	sig = m_alloc_clear (sizeof *sig);
+	if(old_style || opt.force_v3_sigs)
+	  sig->version=3;
+	else if(duration)
+	  sig->version=4;
+	else
+	  sig->version=sk->version;
+	keyid_from_sk (sk, sig->keyid);
+	sig->digest_algo = hash_for (sk->pubkey_algo, sk->version);
+	sig->pubkey_algo = sk->pubkey_algo;
+	if(timestamp)
+	  sig->timestamp = timestamp;
+	else
+	  sig->timestamp = make_timestamp();
+	if(duration)
+	  sig->expiredate = sig->timestamp+duration;
+	sig->sig_class = sigclass;
+
+	md = md_copy (hash);
+
+	if (sig->version >= 4)
+	    build_sig_subpkt_from_sig (sig);
+	mk_notation_and_policy (sig, NULL);
+
+        hash_sigversion_to_magic (md, sig);
+	md_final (md);
+
+	rc = do_sign( sk, sig, md, hash_for (sig->pubkey_algo, sk->version) );
+	md_close (md);
+
+	if( !rc ) { /* and write it */
+            PACKET pkt;
+
+	    init_packet(&pkt);
+	    pkt.pkttype = PKT_SIGNATURE;
+	    pkt.pkt.signature = sig;
+	    rc = build_packet (out, &pkt);
+	    if (!rc && is_status_enabled()) {
+		print_status_sig_created ( sk, sig, status_letter);
+	    }
+	    free_packet (&pkt);
+	    if (rc)
+		log_error ("build signature packet failed: %s\n",
+                           g10_errstr(rc) );
+	}
+	if( rc )
+	    return rc;;
+    }
+
+    return 0;
+}
+
 /****************
  * Sign the files whose names are in FILENAME.
  * If DETACHED has the value true,
  * make a detached signature.  If FILENAMES->d is NULL read from stdin
  * and ignore the detached mode.  Sign the file with all secret keys
  * which can be taken from LOCUSR, if this is NULL, use the default one
- * If ENCRYPT is true, use REMUSER (or ask if it is NULL) to encrypt the
+ * If ENCRYPTFLAG is true, use REMUSER (or ask if it is NULL) to encrypt the
  * signed data for these users.
  * If OUTFILE is not NULL; this file is used for output and the function
  * does not ask for overwrite permission; output is then always
@@ -242,7 +568,7 @@ print_status_sig_created ( PKT_secret_key *sk, PKT_signature *sig, int what )
  */
 int
 sign_file( STRLIST filenames, int detached, STRLIST locusr,
-	   int encrypt, STRLIST remusr, const char *outfile )
+	   int encryptflag, STRLIST remusr, const char *outfile )
 {
     const char *fname;
     armor_filter_context_t afx;
@@ -252,8 +578,6 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     encrypt_filter_context_t efx;
     IOBUF inp = NULL, out = NULL;
     PACKET pkt;
-    PKT_plaintext *pt = NULL;
-    u32 filesize;
     int rc = 0;
     PK_LIST pk_list = NULL;
     SK_LIST sk_list = NULL;
@@ -261,7 +585,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     int multifile = 0;
     int old_style = opt.rfc1991;
     int compr_algo = -1; /* unknown */
-
+    u32 timestamp=0,duration=0;
 
     memset( &afx, 0, sizeof afx);
     memset( &zfx, 0, sizeof zfx);
@@ -277,19 +601,31 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     else
 	fname = NULL;
 
-    if( fname && filenames->next && (!detached || encrypt) )
+    if( fname && filenames->next && (!detached || encryptflag) )
 	log_bug("multiple files can only be detached signed");
 
+    if(opt.ask_sig_expire && !opt.pgp2 && !opt.batch &&
+       !opt.force_v3_sigs && !old_style)
+      duration=ask_expire_interval(1);
+
     if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
 	goto leave;
-    if( !old_style )
+    if( (!old_style && !duration) || opt.pgp2 )
 	old_style = only_old_style( sk_list );
 
-    if( encrypt ) {
+    if(!old_style && opt.pgp2)
+      {
+	log_info(_("you can only sign with PGP 2.x style keys "
+		   "while in --pgp2 mode\n"));
+	log_info(_("this message may not be usable by PGP 2.x\n"));
+	opt.pgp2=0;
+      }
+
+    if( encryptflag ) {
 	if( (rc=build_pk_list( remusr, &pk_list, PUBKEY_USAGE_ENC )) )
 	    goto leave;
 	if( !old_style )
-	    compr_algo = select_algo_from_prefs( pk_list, PREFTYPE_COMPR );
+	    compr_algo = select_algo_from_prefs( pk_list, PREFTYPE_ZIP );
     }
 
     /* prepare iobufs */
@@ -327,20 +663,13 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if( !multifile )
 	iobuf_push_filter( inp, md_filter, &mfx );
 
-    if( detached && !encrypt && !opt.rfc1991 )
+    if( detached && !encryptflag && !opt.rfc1991 )
 	afx.what = 2;
 
     if( opt.armor && !outfile  )
 	iobuf_push_filter( out, armor_filter, &afx );
-  #ifdef ENABLE_COMMENT_PACKETS
-    else {
-	write_comment( out, "#created by GNUPG v" VERSION " ("
-					    PRINTABLE_OS_NAME ")");
-	if( opt.comment_string )
-	    write_comment( out, opt.comment_string );
-    }
-  #endif
-    if( encrypt ) {
+
+    if( encryptflag ) {
 	efx.pk_list = pk_list;
 	/* fixme: set efx.cfx.datalen if known */
 	iobuf_push_filter( out, encrypt_filter, &efx );
@@ -352,48 +681,18 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	else {
 	    if( old_style
 		|| compr_algo == 1
-		|| (compr_algo == -1 && !encrypt) )
+		|| (compr_algo == -1 && !encryptflag) )
 		zfx.algo = 1; /* use the non optional algorithm */
 	    iobuf_push_filter( out, compress_filter, &zfx );
 	}
     }
 
-    if( !detached && !old_style ) {
-	int skcount=0;
-	/* loop over the secret certificates and build headers
-	 * The specs now say that the data should be bracket by
-	 * the onepass-sig and signature-packet; so we must build it
-	 * here in reverse order */
-	for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next )
-	    skcount++;
-	for( ; skcount; skcount-- ) {
-	    PKT_secret_key *sk;
-	    PKT_onepass_sig *ops;
-	    int i = 0;
-
-	    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next )
-		if( ++i == skcount )
-		    break;
-
-	    sk = sk_rover->sk;
-	    ops = m_alloc_clear( sizeof *ops );
-	    ops->sig_class = opt.textmode && !outfile ? 0x01 : 0x00;
-	    ops->digest_algo = hash_for(sk->pubkey_algo, sk->version);
-	    ops->pubkey_algo = sk->pubkey_algo;
-	    keyid_from_sk( sk, ops->keyid );
-	    ops->last = skcount == 1;
-
-	    init_packet(&pkt);
-	    pkt.pkttype = PKT_ONEPASS_SIG;
-	    pkt.pkt.onepass_sig = ops;
-	    rc = build_packet( out, &pkt );
-	    free_packet( &pkt );
-	    if( rc ) {
-		log_error("build onepass_sig packet failed: %s\n",
-							g10_errstr(rc));
-		goto leave;
-	    }
-	}
+    /* Write the one-pass signature packets if needed */
+    if (!detached && !opt.rfc1991) {
+        rc = write_onepass_sig_packets (sk_list, out,
+                                        opt.textmode && !outfile ? 0x01:0x00);
+        if (rc)
+            goto leave;
     }
 
     /* setup the inner packet */
@@ -429,151 +728,21 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	}
     }
     else {
-	if (!opt.no_literal) {
-	    if( fname || opt.set_filename ) {
-		char *s = make_basename( opt.set_filename ? opt.set_filename : fname );
-		pt = m_alloc( sizeof *pt + strlen(s) - 1 );
-		pt->namelen = strlen(s);
-		memcpy(pt->name, s, pt->namelen );
-		m_free(s);
-	    }
-	    else { /* no filename */
-		pt = m_alloc( sizeof *pt - 1 );
-		pt->namelen = 0;
-	    }
-	}
-
-	if( fname ) {
-	    if( !(filesize = iobuf_get_filelength(inp)) )
-		log_info(_("WARNING: `%s' is an empty file\n"), fname );
-            /* we can't yet encode the length of very large files,
-             * so we switch to partial length encoding in this case */
-            if ( filesize >= IOBUF_FILELENGTH_LIMIT )
-                filesize = 0;
-
-	    /* because the text_filter modifies the length of the
-	     * data, it is not possible to know the used length
-	     * without a double read of the file - to avoid that
-	     * we simple use partial length packets.
-	     */
-	    if( opt.textmode && !outfile )
-		filesize = 0;
-	}
-	else
-	    filesize = opt.set_filesize ? opt.set_filesize : 0; /* stdin */
-
-	if (!opt.no_literal) {
-	    pt->timestamp = make_timestamp();
-	    pt->mode = opt.textmode && !outfile ? 't':'b';
-	    pt->len = filesize;
-	    pt->new_ctb = !pt->len && !opt.rfc1991;
-	    pt->buf = inp;
-	    pkt.pkttype = PKT_PLAINTEXT;
-	    pkt.pkt.plaintext = pt;
-	    /*cfx.datalen = filesize? calc_packet_length( &pkt ) : 0;*/
-	    if( (rc = build_packet( out, &pkt )) )
-		log_error("build_packet(PLAINTEXT) failed: %s\n", g10_errstr(rc) );
-	    pt->buf = NULL;
-	}
-	else {
-	    byte copy_buffer[4096];
-	    int  bytes_copied;
-	    while ((bytes_copied = iobuf_read(inp, copy_buffer, 4096)) != -1)
-		if (iobuf_write(out, copy_buffer, bytes_copied) == -1) {
-		    rc = G10ERR_WRITE_FILE;
-		    log_error("copying input to output failed: %s\n", g10_errstr(rc));
-		    break;
-		}
-	    memset(copy_buffer, 0, 4096); /* burn buffer */
-	}
+        rc = write_plaintext_packet (out, inp, fname,
+                                     opt.textmode && !outfile ? 't':'b');
     }
 
-    /* catch errors from above blocks */
+    /* catch errors from above */
     if (rc)
 	goto leave;
 
-    /* loop over the secret certificates */
-    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
-	PKT_secret_key *sk;
-	PKT_signature *sig;
-	MD_HANDLE md;
-
-	sk = sk_rover->sk;
-
-	/* build the signature packet */
-	/* fixme: this code is partly duplicated in make_keysig_packet */
-	sig = m_alloc_clear( sizeof *sig );
-	sig->version = old_style || opt.force_v3_sigs ? 3 : sk->version;
-	keyid_from_sk( sk, sig->keyid );
-	sig->digest_algo = hash_for(sk->pubkey_algo, sk->version);
-	sig->pubkey_algo = sk->pubkey_algo;
-	sig->timestamp = make_timestamp();
-	sig->sig_class = opt.textmode && !outfile? 0x01 : 0x00;
-
-	md = md_copy( mfx.md );
-
-	if( sig->version >= 4 ) {
-	    build_sig_subpkt_from_sig( sig );
-	    md_putc( md, sig->version );
-	}
-
-	mk_notation_and_policy( sig );
-
-	md_putc( md, sig->sig_class );
-	if( sig->version < 4 ) {
-	    u32 a = sig->timestamp;
-	    md_putc( md, (a >> 24) & 0xff );
-	    md_putc( md, (a >> 16) & 0xff );
-	    md_putc( md, (a >>	8) & 0xff );
-	    md_putc( md,  a	   & 0xff );
-	}
-	else {
-	    byte buf[6];
-	    size_t n;
-
-	    md_putc( md, sig->pubkey_algo );
-	    md_putc( md, sig->digest_algo );
-	    if( sig->hashed_data ) {
-		n = (sig->hashed_data[0] << 8) | sig->hashed_data[1];
-		md_write( md, sig->hashed_data, n+2 );
-		n += 6;
-	    }
-	    else {
-		md_putc( md, 0 );  /* always hash the length of the subpacket*/
-		md_putc( md, 0 );
-		n = 6;
-	    }
-	    /* add some magic */
-	    buf[0] = sig->version;
-	    buf[1] = 0xff;
-	    buf[2] = n >> 24; /* hmmm, n is only 16 bit, so this is always 0 */
-	    buf[3] = n >> 16;
-	    buf[4] = n >>  8;
-	    buf[5] = n;
-	    md_write( md, buf, 6 );
-
-	}
-	md_final( md );
-
-	rc = do_sign( sk, sig, md, hash_for(sig->pubkey_algo, sk->version) );
-	md_close( md );
-
-	if( !rc ) { /* and write it */
-	    init_packet(&pkt);
-	    pkt.pkttype = PKT_SIGNATURE;
-	    pkt.pkt.signature = sig;
-	    rc = build_packet( out, &pkt );
-	    if( !rc && is_status_enabled() ) {
-		print_status_sig_created ( sk, sig, detached ? 'D':'S');
-	    }
-	    free_packet( &pkt );
-	    if( rc )
-		log_error("build signature packet failed: %s\n", g10_errstr(rc) );
-	}
-	if( rc )
-	    goto leave;
-
-    }
+    /* write the signatures */
+    rc = write_signature_packets (sk_list, out, mfx.md,
+                                  opt.textmode && !outfile? 0x01 : 0x00,
+				  timestamp, duration,
+                                  old_style, detached ? 'D':'S');
+    if( rc )
+        goto leave;
 
 
   leave:
@@ -581,7 +750,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	iobuf_cancel(out);
     else {
 	iobuf_close(out);
-        if (encrypt)
+        if (encryptflag)
             write_status( STATUS_END_ENCRYPTION );
     }
     iobuf_close(inp);
@@ -608,15 +777,28 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     SK_LIST sk_rover = NULL;
     int old_style = opt.rfc1991;
     int only_md5 = 0;
+    u32 timestamp=0,duration=0;
 
     memset( &afx, 0, sizeof afx);
     init_packet( &pkt );
 
+    if(opt.ask_sig_expire && !opt.pgp2 && !opt.batch &&
+       !opt.force_v3_sigs && !old_style)
+      duration=ask_expire_interval(1);
+
     if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
 	goto leave;
-    if( !old_style )
+    if( (!old_style && !duration) || opt.pgp2 )
 	old_style = only_old_style( sk_list );
 
+    if(!old_style && opt.pgp2)
+      {
+	log_info(_("you can only clearsign with PGP 2.x style keys "
+		   "while in --pgp2 mode\n"));
+	log_info(_("this message may not be usable by PGP 2.x\n"));
+	opt.pgp2=0;
+      }
+
     /* prepare iobufs */
     if( !(inp = iobuf_open(fname)) ) {
 	log_error("can't open %s: %s\n", fname? fname: "[stdin]",
@@ -649,9 +831,7 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	}
     }
 
-    if( old_style && only_md5 )
-	iobuf_writestr(out, LF );
-    else {
+    if( !(old_style && only_md5) ) {
 	const char *s;
 	int any = 0;
 	byte hashs_seen[256];
@@ -675,12 +855,12 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	}
 	assert(any);
 	iobuf_writestr(out, LF );
-	if( opt.not_dash_escaped )
-	    iobuf_writestr( out,
-		"NotDashEscaped: You need GnuPG to verify this message" LF );
-	iobuf_writestr(out, LF );
     }
 
+    if( opt.not_dash_escaped )
+      iobuf_writestr( out,
+		  "NotDashEscaped: You need GnuPG to verify this message" LF );
+    iobuf_writestr(out, LF );
 
     textmd = md_open(0, 0);
     for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
@@ -697,87 +877,11 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     afx.what = 2;
     iobuf_push_filter( out, armor_filter, &afx );
 
-    /* loop over the secret certificates */
-    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
-	PKT_secret_key *sk;
-	PKT_signature *sig;
-	MD_HANDLE md;
-
-	sk = sk_rover->sk;
-
-	/* build the signature packet */
-	/* fixme: this code is duplicated above */
-	sig = m_alloc_clear( sizeof *sig );
-	sig->version = old_style || opt.force_v3_sigs ? 3 : sk->version;
-	keyid_from_sk( sk, sig->keyid );
-	sig->digest_algo = hash_for(sk->pubkey_algo, sk->version);
-	sig->pubkey_algo = sk->pubkey_algo;
-	sig->timestamp = make_timestamp();
-	sig->sig_class = 0x01;
-
-	md = md_copy( textmd );
-	if( sig->version >= 4 ) {
-	    build_sig_subpkt_from_sig( sig );
-	    md_putc( md, sig->version );
-	}
-
-	mk_notation_and_policy( sig );
-
-	md_putc( md, sig->sig_class );
-	if( sig->version < 4 ) {
-	    u32 a = sig->timestamp;
-	    md_putc( md, (a >> 24) & 0xff );
-	    md_putc( md, (a >> 16) & 0xff );
-	    md_putc( md, (a >>	8) & 0xff );
-	    md_putc( md,  a	   & 0xff );
-	}
-	else {
-	    byte buf[6];
-	    size_t n;
-
-	    md_putc( md, sig->pubkey_algo );
-	    md_putc( md, sig->digest_algo );
-	    if( sig->hashed_data ) {
-		n = (sig->hashed_data[0] << 8) | sig->hashed_data[1];
-		md_write( md, sig->hashed_data, n+2 );
-		n += 6;
-	    }
-	    else {
-		md_putc( md, 0 );  /* always hash the length of the subpacket*/
-		md_putc( md, 0 );
-		n = 6;
-	    }
-	    /* add some magic */
-	    buf[0] = sig->version;
-	    buf[1] = 0xff;
-	    buf[2] = n >> 24; /* hmmm, n is only 16 bit, so this is always 0 */
-	    buf[3] = n >> 16;
-	    buf[4] = n >>  8;
-	    buf[5] = n;
-	    md_write( md, buf, 6 );
-
-	}
-	md_final( md );
-
-	rc = do_sign( sk, sig, md, hash_for(sig->pubkey_algo, sk->version) );
-	md_close( md );
-
-	if( !rc ) { /* and write it */
-	    init_packet(&pkt);
-	    pkt.pkttype = PKT_SIGNATURE;
-	    pkt.pkt.signature = sig;
-	    rc = build_packet( out, &pkt );
-	    if( !rc && is_status_enabled() ) {
-		print_status_sig_created ( sk, sig, 'C');
-	    }
-	    free_packet( &pkt );
-	    if( rc )
-		log_error("build signature packet failed: %s\n", g10_errstr(rc) );
-	}
-	if( rc )
-	    goto leave;
-    }
-
+    /* write the signatures */
+    rc = write_signature_packets (sk_list, out, textmd, 0x01,
+				  timestamp, duration, old_style, 'C');
+    if( rc )
+        goto leave;
 
   leave:
     if( rc )
@@ -790,18 +894,177 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     return rc;
 }
 
+/*
+ * Sign and conventionally encrypt the given file.
+ * FIXME: Far too much code is duplicated - revamp the whole file.
+ */
+int
+sign_symencrypt_file (const char *fname, STRLIST locusr)
+{
+    armor_filter_context_t afx;
+    compress_filter_context_t zfx;
+    md_filter_context_t mfx;
+    text_filter_context_t tfx;
+    cipher_filter_context_t cfx;
+    IOBUF inp = NULL, out = NULL;
+    PACKET pkt;
+    STRING2KEY *s2k = NULL;
+    int rc = 0;
+    SK_LIST sk_list = NULL;
+    SK_LIST sk_rover = NULL;
+    int old_style = opt.rfc1991;
+    int compr_algo = -1; /* unknown */
+    int algo;
+    u32 timestamp=0,duration=0;
+
+    memset( &afx, 0, sizeof afx);
+    memset( &zfx, 0, sizeof zfx);
+    memset( &mfx, 0, sizeof mfx);
+    memset( &tfx, 0, sizeof tfx);
+    memset( &cfx, 0, sizeof cfx);
+    init_packet( &pkt );
+
+    if(opt.ask_sig_expire && !opt.batch && !opt.force_v3_sigs && !old_style)
+      duration=ask_expire_interval(1);
+
+    rc = build_sk_list (locusr, &sk_list, 1, PUBKEY_USAGE_SIG);
+    if (rc) 
+	goto leave;
+    if( !old_style && !duration )
+	old_style = only_old_style( sk_list );
+
+    /* prepare iobufs */
+    inp = iobuf_open(fname);
+    if( !inp ) {
+	log_error("can't open %s: %s\n", fname? fname: "[stdin]",
+					strerror(errno) );
+	rc = G10ERR_OPEN_FILE;
+	goto leave;
+    }
+
+    /* prepare key */
+    s2k = m_alloc_clear( sizeof *s2k );
+    s2k->mode = opt.rfc1991? 0:opt.s2k_mode;
+    s2k->hash_algo = opt.def_digest_algo ? opt.def_digest_algo
+	                                 : opt.s2k_digest_algo;
+
+    algo = opt.def_cipher_algo ? opt.def_cipher_algo : opt.s2k_cipher_algo;
+    if (!opt.quiet || !opt.batch)
+        log_info (_("%s encryption will be used\n"),
+		    cipher_algo_to_string(algo) );
+    cfx.dek = passphrase_to_dek( NULL, 0, algo, s2k, 2, NULL );
+
+    if (!cfx.dek || !cfx.dek->keylen) {
+        rc = G10ERR_PASSPHRASE;
+        log_error(_("error creating passphrase: %s\n"), g10_errstr(rc) );
+        goto leave;
+    }
+
+    /* now create the outfile */
+    rc = open_outfile (fname, opt.armor? 1:0, &out);
+    if (rc)
+	goto leave;
+
+    /* prepare to calculate the MD over the input */
+    if (opt.textmode)
+	iobuf_push_filter (inp, text_filter, &tfx);
+    mfx.md = md_open(0, 0);
+
+    for (sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next) {
+	PKT_secret_key *sk = sk_rover->sk;
+	md_enable (mfx.md, hash_for (sk->pubkey_algo, sk->version ));
+    }
+
+    iobuf_push_filter (inp, md_filter, &mfx);
+
+    /* Push armor output filter */
+    if (opt.armor)
+	iobuf_push_filter (out, armor_filter, &afx);
+
+    /* Write the symmetric key packet */
+    /*(current filters: armor)*/
+    if (!opt.rfc1991) {
+	PKT_symkey_enc *enc = m_alloc_clear( sizeof *enc );
+	enc->version = 4;
+	enc->cipher_algo = cfx.dek->algo;
+	enc->s2k = *s2k;
+	pkt.pkttype = PKT_SYMKEY_ENC;
+	pkt.pkt.symkey_enc = enc;
+	if( (rc = build_packet( out, &pkt )) )
+	    log_error("build symkey packet failed: %s\n", g10_errstr(rc) );
+	m_free(enc);
+    }
+
+    /* Push the encryption filter */
+    iobuf_push_filter( out, cipher_filter, &cfx );
+
+    /* Push the Zip filter */
+    if (opt.compress) {
+	if (!compr_algo)
+	    ; /* don't use compression */
+	else {
+	    if( old_style || compr_algo == 1 )
+		zfx.algo = 1; /* use the non optional algorithm */
+	    iobuf_push_filter( out, compress_filter, &zfx );
+	}
+    }
+
+    /* Write the one-pass signature packets */
+    /*(current filters: zip - encrypt - armor)*/
+    if (!opt.rfc1991) {
+        rc = write_onepass_sig_packets (sk_list, out,
+                                        opt.textmode? 0x01:0x00);
+        if (rc)
+            goto leave;
+    }
+
+    /* Pipe data through all filters; i.e. write the signed stuff */
+    /*(current filters: zip - encrypt - armor)*/
+    rc = write_plaintext_packet (out, inp, fname, opt.textmode ? 't':'b');
+    if (rc)
+	goto leave;
+    
+    /* Write the signatures */
+    /*(current filters: zip - encrypt - armor)*/
+    rc = write_signature_packets (sk_list, out, mfx.md,
+				  opt.textmode? 0x01 : 0x00,
+				  timestamp, duration,
+                                  old_style, 'S');
+    if( rc )
+        goto leave;
+
+
+  leave:
+    if( rc )
+	iobuf_cancel(out);
+    else {
+	iobuf_close(out);
+        write_status( STATUS_END_ENCRYPTION );
+    }
+    iobuf_close(inp);
+    release_sk_list( sk_list );
+    md_close( mfx.md );
+    m_free(cfx.dek);
+    m_free(s2k);
+    return rc;
+}
+
 
 /****************
- * Create a signature packet for the given public key certificate
- * and the user id and return it in ret_sig. User signature class SIGCLASS
- * user-id is not used (and may be NULL if sigclass is 0x20)
- * If digest_algo is 0 the function selects an appropriate one.
- */
+ * Create a signature packet for the given public key certificate and
+ * the user id and return it in ret_sig. User signature class SIGCLASS
+ * user-id is not used (and may be NULL if sigclass is 0x20) If
+ * DIGEST_ALGO is 0 the function selects an appropriate one.
+ * SIGVERSION gives the minimal required signature packet version;
+ * this is needed so that special properties like local sign are not
+ * applied (actually: dropped) when a v3 key is used.  TIMESTAMP is
+ * the timestamp to use for the signature. 0 means "now" */
 int
 make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 		    PKT_user_id *uid, PKT_public_key *subpk,
 		    PKT_secret_key *sk,
 		    int sigclass, int digest_algo,
+                    int sigversion, u32 timestamp, u32 duration,
 		    int (*mksubpkt)(PKT_signature *, void *), void *opaque
 		   )
 {
@@ -812,12 +1075,35 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
     assert( (sigclass >= 0x10 && sigclass <= 0x13)
 	    || sigclass == 0x20 || sigclass == 0x18
 	    || sigclass == 0x30 || sigclass == 0x28 );
+
+    if (opt.force_v4_certs)
+        sigversion = 4;
+
+    if (sigversion < sk->version)
+        sigversion = sk->version;
+
+    /* If you are making a signature on a v4 key using your v3 key, it
+       doesn't make sense to generate a v3 sig.  After all, no v3-only
+       PGP implementation could understand the v4 key in the first
+       place. */
+    if (sigversion < pk->version)
+        sigversion = pk->version;
+
     if( !digest_algo ) {
 	switch( sk->pubkey_algo ) {
-	  case PUBKEY_ALGO_DSA: digest_algo = DIGEST_ALGO_SHA1; break;
+	  case PUBKEY_ALGO_DSA:
+            digest_algo = DIGEST_ALGO_SHA1; 
+            break;
 	  case PUBKEY_ALGO_RSA_S:
-	  case PUBKEY_ALGO_RSA: digest_algo = DIGEST_ALGO_MD5; break;
-	  default:		digest_algo = DIGEST_ALGO_RMD160; break;
+	  case PUBKEY_ALGO_RSA:
+            if (opt.force_v4_certs || sk->version > 3)
+              digest_algo = DIGEST_ALGO_SHA1;
+            else
+              digest_algo = DIGEST_ALGO_MD5;
+            break;
+	  default:
+            digest_algo = DIGEST_ALGO_RMD160;
+            break;
 	}
     }
     md = md_open( digest_algo, 0 );
@@ -828,24 +1114,22 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 	hash_public_key( md, subpk );
     }
     else if( sigclass != 0x20 ) {
-	if( sk->version >=4 ) {
-	    byte buf[5];
-	    buf[0] = 0xb4;	      /* indicates a userid packet */
-	    buf[1] = uid->len >> 24;  /* always use 4 length bytes */
-	    buf[2] = uid->len >> 16;
-	    buf[3] = uid->len >>  8;
-	    buf[4] = uid->len;
-	    md_write( md, buf, 5 );
-	}
-	md_write( md, uid->name, uid->len );
+        hash_uid (md, sigversion, uid);
     }
     /* and make the signature packet */
     sig = m_alloc_clear( sizeof *sig );
-    sig->version = sk->version;
+    sig->version = sigversion;
+    sig->flags.exportable=1;
+    sig->flags.revocable=1;
     keyid_from_sk( sk, sig->keyid );
     sig->pubkey_algo = sk->pubkey_algo;
     sig->digest_algo = digest_algo;
-    sig->timestamp = make_timestamp();
+    if(timestamp)
+      sig->timestamp=timestamp;
+    else
+      sig->timestamp=make_timestamp();
+    if(duration)
+      sig->expiredate=sig->timestamp+duration;
     sig->sig_class = sigclass;
     if( sig->version >= 4 )
 	build_sig_subpkt_from_sig( sig );
@@ -854,43 +1138,8 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 	rc = (*mksubpkt)( sig, opaque );
 
     if( !rc ) {
-	mk_notation_and_policy( sig );
-	if( sig->version >= 4 )
-	    md_putc( md, sig->version );
-	md_putc( md, sig->sig_class );
-	if( sig->version < 4 ) {
-	    u32 a = sig->timestamp;
-	    md_putc( md, (a >> 24) & 0xff );
-	    md_putc( md, (a >> 16) & 0xff );
-	    md_putc( md, (a >>	8) & 0xff );
-	    md_putc( md,  a	   & 0xff );
-	}
-	else {
-	    byte buf[6];
-	    size_t n;
-
-	    md_putc( md, sig->pubkey_algo );
-	    md_putc( md, sig->digest_algo );
-	    if( sig->hashed_data ) {
-		n = (sig->hashed_data[0] << 8) | sig->hashed_data[1];
-		md_write( md, sig->hashed_data, n+2 );
-		n += 6;
-	    }
-	    else {
-		md_putc( md, 0 );  /* always hash the length of the subpacket*/
-		md_putc( md, 0 );
-		n = 6;
-	    }
-	    /* add some magic */
-	    buf[0] = sig->version;
-	    buf[1] = 0xff;
-	    buf[2] = n >> 24; /* hmmm, n is only 16 bit, so this is always 0 */
-	    buf[3] = n >> 16;
-	    buf[4] = n >>  8;
-	    buf[5] = n;
-	    md_write( md, buf, 6 );
-
-	}
+	mk_notation_and_policy( sig, pk );
+        hash_sigversion_to_magic (md, sig);
 	md_final(md);
 
 	rc = complete_sig( sig, sk, md );
@@ -905,3 +1154,65 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 }
 
 
+
+/****************
+ * Create a new signature packet based on an existing one.
+ * Only user ID signatures are supported for now.
+ * TODO: Merge this with make_keysig_packet.
+ */
+int
+update_keysig_packet( PKT_signature **ret_sig,
+                      PKT_signature *orig_sig,
+                      PKT_public_key *pk,
+                      PKT_user_id *uid, 
+                      PKT_secret_key *sk,
+                      int (*mksubpkt)(PKT_signature *, void *),
+                      void *opaque
+		   )
+{
+    PKT_signature *sig;
+    int rc=0;
+    MD_HANDLE md;
+
+    if (!orig_sig || !pk || !uid || !sk)
+        return G10ERR_GENERAL;
+    if (orig_sig->sig_class < 0x10 || orig_sig->sig_class > 0x13 )
+        return G10ERR_GENERAL;
+
+    md = md_open( orig_sig->digest_algo, 0 );
+
+    /* hash the public key certificate and the user id */
+    hash_public_key( md, pk );
+    hash_uid (md, orig_sig->version, uid);
+
+    /* create a new signature packet */
+    sig = copy_signature (NULL, orig_sig);
+    if ( sig->version >= 4 && mksubpkt)
+	rc = (*mksubpkt)(sig, opaque);
+
+    /* we increase the timestamp by one second so that a future import
+       of this key will replace the existing one.  We also make sure that
+       we don't create a timestamp in the future */
+    sig->timestamp++; 
+    while (sig->timestamp >= make_timestamp())
+        sleep (1);
+    /* put the updated timestamp back into the data */
+    if( sig->version >= 4 )
+	build_sig_subpkt_from_sig( sig );
+
+    if (!rc) {
+        hash_sigversion_to_magic (md, sig);
+	md_final(md);
+
+	rc = complete_sig( sig, sk, md );
+    }
+
+    md_close (md);
+    if( rc )
+	free_seckey_enc (sig);
+    else
+	*ret_sig = sig;
+    return rc;
+}
+
+
