@@ -1,5 +1,5 @@
 /* status.c
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -84,7 +84,7 @@ get_status_string ( int no )
       case STATUS_LEAVE  : s = "LEAVE"; break;
       case STATUS_ABORT  : s = "ABORT"; break;
       case STATUS_GOODSIG: s = "GOODSIG"; break;
-      case STATUS_SIGEXPIRED: s = "SIGEXPIRED"; break;
+      case STATUS_KEYEXPIRED: s = "KEYEXPIRED"; break;
       case STATUS_KEYREVOKED: s = "KEYREVOKED"; break;
       case STATUS_BADSIG : s = "BADSIG"; break;
       case STATUS_ERRSIG : s = "ERRSIG"; break;
@@ -140,6 +140,12 @@ get_status_string ( int no )
       case STATUS_KEY_CREATED    : s = "KEY_CREATED"; break;
       case STATUS_USERID_HINT    : s = "USERID_HINT"; break;
       case STATUS_UNEXPECTED     : s = "UNEXPECTED"; break;
+      case STATUS_INV_RECP       : s = "INV_RECP"; break;
+      case STATUS_NO_RECP        : s = "NO_RECP"; break;
+      case STATUS_ALREADY_SIGNED : s = "ALREADY_SIGNED"; break;
+      case STATUS_SIGEXPIRED     : s = "SIGEXPIRED deprecated-use-keyexpired-instead"; break;
+      case STATUS_EXPSIG         : s = "EXPSIG"; break;
+      case STATUS_EXPKEYSIG      : s = "EXPKEYSIG"; break;
       default: s = "?"; break;
     }
     return s;
@@ -212,28 +218,43 @@ write_status_text ( int no, const char *text)
 
 
 /*
- * Write a status line with a buffer using %XX escapes.  
- * If WRAP is > 0 wrap the line after this length. 
+ * Write a status line with a buffer using %XX escapes.  If WRAP is >
+ * 0 wrap the line after this length.  If STRING is not NULL it will
+ * be prepended to the buffer, no escaping is done for string.
+ * A wrap of -1 forces spaces not to be encoded as %20.
  */
 void
-write_status_buffer ( int no, const char *buffer, size_t len, int wrap )
+write_status_text_and_buffer ( int no, const char *string,
+                               const char *buffer, size_t len, int wrap )
 {
     const char *s, *text;
-    int esc;
+    int esc, first;
+    int lower_limit = ' ';
     size_t n, count, dowrap;
 
     if( !statusfp )
 	return;  /* not enabled */
+    
+    if (wrap == -1) {
+        lower_limit--;
+        wrap = 0;
+    }
 
     text = get_status_string (no);
-    count = dowrap = 1;
+    count = dowrap = first = 1;
     do {
         if (dowrap) {
             fprintf (statusfp, "[GNUPG:] %s ", text );
             count = dowrap = 0;
+            if (first && string) {
+                fputs (string, statusfp);
+                count += strlen (string);
+            }
+            first = 0;
         }
         for (esc=0, s=buffer, n=len; n && !esc; s++, n-- ) {
-            if ( *s == '%' || *(const byte*)s <= ' ' ) 
+            if ( *s == '%' || *(const byte*)s <= lower_limit 
+                           || *(const byte*)s == 127 ) 
                 esc = 1;
             if ( wrap && ++count > wrap ) {
                 dowrap=1;
@@ -259,6 +280,13 @@ write_status_buffer ( int no, const char *buffer, size_t len, int wrap )
     fflush (statusfp);
 }
 
+void
+write_status_buffer ( int no, const char *buffer, size_t len, int wrap )
+{
+    write_status_text_and_buffer (no, NULL, buffer, len, wrap);
+}
+
+
 
 #ifdef USE_SHM_COPROCESSING
 
