@@ -20,6 +20,10 @@
 #ifndef G10_UTIL_H
 #define G10_UTIL_H
 
+#if defined (__MINGW32__) || defined (__CYGWIN32__)
+# include <stdarg.h>
+#endif
+
 #include "types.h"
 #include "errors.h"
 #include "types.h"
@@ -146,6 +150,7 @@ char *make_filename( const char *first_part, ... );
 int compare_filenames( const char *a, const char *b );
 const char *print_fname_stdin( const char *s );
 const char *print_fname_stdout( const char *s );
+int  is_file_compressed(const char *s, int *r_status);
 
 
 /*-- miscutil.c --*/
@@ -157,7 +162,9 @@ const char *strtimestamp( u32 stamp ); /* GMT */
 const char *asctimestamp( u32 stamp ); /* localized */
 void print_string( FILE *fp, const byte *p, size_t n, int delim );
 void  print_utf8_string( FILE *fp, const byte *p, size_t n );
+void  print_utf8_string2( FILE *fp, const byte *p, size_t n, int delim);
 char *make_printable_string( const byte *p, size_t n, int delim );
+int answer_is_yes_no_default( const char *s, int def_answer );
 int answer_is_yes( const char *s );
 int answer_is_yes_no_quit( const char *s );
 
@@ -184,7 +191,7 @@ int string_count_chr( const char *string, int c );
 int set_native_charset( const char *newset );
 const char* get_native_charset(void);
 char *native_to_utf8( const char *string );
-char *utf8_to_native( const char *string, size_t length );
+char *utf8_to_native( const char *string, size_t length, int delim);
 int  check_utf8_string( const char *string );
 
 int ascii_isupper (int c);
@@ -203,6 +210,9 @@ char *stpcpy(char *a,const char *b);
 #ifndef HAVE_STRLWR
 char *strlwr(char *a);
 #endif
+#ifndef HAVE_STRSEP
+char *strsep (char **stringp, const char *delim);
+#endif
 #ifndef HAVE_STRCASECMP
 int strcasecmp( const char *, const char *b);
 #endif
@@ -216,10 +226,13 @@ int strcasecmp( const char *, const char *b);
   #define stricmp(a,b)	 strcasecmp( (a), (b) )
 #endif
 
-#ifdef __MINGW32__
+#if defined (__MINGW32__) || defined (__CYGWIN32__)
 /*-- w32reg.c --*/
 char *read_w32_registry_string( const char *root,
 				const char *dir, const char *name );
+int write_w32_registry_string(const char *root, const char *dir,
+                              const char *name, const char *value);
+
 /*-- strgutil.c --*/
 int vasprintf ( char **result, const char *format, va_list args);
 #endif
@@ -243,19 +256,22 @@ int vasprintf ( char **result, const char *format, va_list args);
 
 /******* RISC OS stuff ***********/
 #ifdef __riscos__
-FILE *riscos_fopen(const char *filename, const char *mode);
-int riscos_open(const char *filename, int oflag, ...);
-int riscos_fstat(int fildes, struct stat *buf);
+pid_t riscos_getpid(void);
+int riscos_kill(pid_t pid, int sig);
+int riscos_access(const char *path, int amode);
 int fdopenfile(const char *filename, const int allow_write);
 void close_fds(void);
 int renamefile(const char *old, const char *new);
 char *gstrans(const char *old);
 void not_implemented(const char *feature);
-void set_filetype(const char *filename, const int type);
+#ifdef DEBUG
+void dump_fdlist(void);
+void list_openfiles(void);
+#endif
 #ifndef __RISCOS__C__
-  #define fopen riscos_fopen
-  #define open riscos_open
-  #define fstat riscos_fstat
+  #define getpid riscos_getpid
+  #define kill(a,b) riscos_kill((a),(b))
+  #define access(a,b) riscos_access((a),(b))
 #endif /* !__RISCOS__C__ */
 #endif /* __riscos__ */
 
