@@ -34,7 +34,7 @@
 #endif
 #ifdef __riscos__
 #include <kernel.h>
-#include <sys/swis.h>
+#include <swis.h>
 #endif /* __riscos__ */
 
 #include "memory.h"
@@ -208,7 +208,20 @@ direct_open (const char *fname, const char *mode)
     else {
         oflag = O_RDONLY;
     }
+#ifndef __riscos__
     return open (fname, oflag, cflag );
+#else
+    {
+        struct stat buf;
+        int rc = stat( fname, &buf );
+        
+        /* Don't allow iobufs on directories */
+        if( !rc && S_ISDIR(buf.st_mode) && !S_ISREG(buf.st_mode) )
+            return __set_errno( EISDIR );
+        else
+            return open( fname, oflag, cflag );
+    }
+#endif
 #endif
 }
 
@@ -230,7 +243,7 @@ fd_cache_close (const char *fname, FILEP_OR_FD fp)
         close(fp);
 #endif
         if( DBG_IOBUF )
-            log_debug ("fd_cache_close (%p) real\n", fp);
+            log_debug ("fd_cache_close (%p) real\n", (void*)fp);
         return;
     }
     /* try to reuse a slot */
@@ -1097,7 +1110,7 @@ iobuf_fdopen( int fd, const char *mode )
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
 	log_debug("iobuf-%d.%d: fdopen `%s'\n", a->no, a->subno, fcx->fname );
-
+    iobuf_ioctl (a,3,1,NULL); /* disable fd caching */
     return a;
 }
 
@@ -1121,11 +1134,10 @@ iobuf_sockopen ( int fd, const char *mode )
     sock_filter( scx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
 	log_debug("iobuf-%d.%d: sockopen `%s'\n", a->no, a->subno, scx->fname);
+    iobuf_ioctl (a,3,1,NULL); /* disable fd caching */ 
 #else
     a = iobuf_fdopen (fd, mode);
 #endif
-    if (a)
-        iobuf_ioctl (a,3,1,NULL); /* disable fd caching */ 
     return a;
 }
 
