@@ -32,6 +32,9 @@
   #ifdef USE_CAPABILITIES
     #include <sys/capability.h>
   #endif
+  #ifdef HAVE_PLOCK
+    #include <sys/lock.h>
+  #endif
 #endif
 
 #include "types.h"
@@ -80,8 +83,12 @@ static int suspend_warning;
 static void
 print_warn(void)
 {
-    if( !no_warning )
-	log_info(_("Warning: using insecure memory!\n"));
+  if (!no_warning)
+    {
+      log_info(_("Warning: using insecure memory!\n"));
+      log_info(_("please see http://www.gnupg.org/faq.html "
+		 "for more information\n"));
+    }
 }
 
 
@@ -120,6 +127,13 @@ lock_pool( void *p, size_t n )
     uid = getuid();
 
   #ifdef HAVE_BROKEN_MLOCK
+    /* ick. but at least we get secured memory. about to lock
+       entire data segment. */
+  #ifdef HAVE_PLOCK
+    err = plock( DATLOCK );
+    if( err && errno )
+        err = errno;
+#else /*!HAVE_PLOCK*/
     if( uid ) {
 	errno = EPERM;
 	err = errno;
@@ -129,6 +143,7 @@ lock_pool( void *p, size_t n )
 	if( err && errno )
 	    err = errno;
     }
+  #endif /*!HAVE_PLOCK*/
   #else
     err = mlock( p, n );
     if( err && errno )
@@ -168,6 +183,11 @@ lock_pool( void *p, size_t n )
     /* It does not make sense to print such a warning, given the fact that 
      * this whole Windows !@#$% and their user base are inherently insecure
      */
+  #elif defined (__riscos__)
+    /* no virtual memory on RISC OS, so no pages are swapped to disc,
+     * besides we don't have mmap, so we don't use it! ;-)
+     * But don't complain, as explained above.
+     */
   #else
     log_info("Please note that you don't have secure memory on this system\n");
   #endif
@@ -267,6 +287,7 @@ void
 secmem_init( size_t n )
 {
     if( !n ) {
+#ifndef __riscos__
       #ifdef USE_CAPABILITIES
 	/* drop all capabilities */
 	cap_set_proc( cap_from_text("all-eip") );
@@ -281,6 +302,7 @@ secmem_init( size_t n )
 		log_fatal("failed to drop setuid\n" );
 	}
       #endif
+#endif /* !__riscos__ */
     }
     else {
 	if( n < DEFAULT_POOLSIZE )
